/****************************************************************************************//**
* \file				avagoMinipodCtrl.c
*
* \brief This file contains the user driver to control the 6 AVAGO miniPOD of the AMC40 board.
* 
* \author PYD : 17/4/2012
* \version 0.1
* \date 17/4/2012
* \copyright (c) Copyright CERN for the benefit of the LHCb Collaboration.
*    Distributed under the terms of the GNU General Public Licence V3.
*    ana.py script is free and open source software.
*
* CHANGELOG
*    PYD : 17/4/2012 initial version
*    PYD : 12/6/2015 several i2c buses implemented
*    PYD : 07/01/2016 adaptation for PCIe40
*    PYD : 15/09/2016 add multi boards
*    PYD : 01/10/2018 remove code for ES12 and ES3 versions
*//********************************************************************************************/
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <math.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <linux/i2c-dev.h>
#include <systemConfig.h>
#include <i2cDriver.h>
#include <avagoMinipodCtrl.h>

 typedef struct {
	int reg;
	int quart;
	} quartPlace;
 /**
 *  \var mpod_vodQuarters
 *  \brief Channels are organized from 0 to 11. Data of channel X are at index X in the array.
 *
 *     The channel data are:
 *
 *       The register number     : tab[chnId][0]
 *
 *       The associated quarter  : tab[chnId][1]  (0 means low, 1 means high quarter)
*/

static quartPlace mpod_vodQuarters[12] = {
  {MPOD_RXVOD1, 0},{MPOD_RXVOD1, 1},{MPOD_RXVOD2, 0},{MPOD_RXVOD2, 1},{MPOD_RXVOD3, 0},{MPOD_RXVOD3, 1},
  {MPOD_RXVOD4, 0},{MPOD_RXVOD4, 1},{MPOD_RXVOD5, 0},{MPOD_RXVOD5, 1},{MPOD_RXVOD6, 0},{MPOD_RXVOD6, 1}
 };
 /**
 *  \var mpod_deamphQuarters
 *  \brief Channels are organized from 0 to 11. Data of channel X are at index X in the array.
 *
 *     The channel data are:
 *
 *       The register number     : tab[chnId][0]
 *
 *       The associated quarter  : tab[chnId][1]  (0 means low, 1 means high quarter)
*/
static quartPlace mpod_deamphQuarters[12] = {
  {MPOD_RXDEAMPH1, 0},{MPOD_RXDEAMPH1, 1},{MPOD_RXDEAMPH2, 0},{MPOD_RXDEAMPH2, 1},{MPOD_RXDEAMPH3, 0},{MPOD_RXDEAMPH3, 1},
  {MPOD_RXDEAMPH4, 0},{MPOD_RXDEAMPH4, 1},{MPOD_RXDEAMPH5, 0},{MPOD_RXDEAMPH5, 1},{MPOD_RXDEAMPH6, 0},{MPOD_RXDEAMPH6, 1}
 };
 /**
 *  \var mpod_equalzQuarters
 *  \brief Channels are organized from 0 to 11. Data of channel X are at index X in the array.
 *
 *     The channel data are:
 *
 *       The register number     : tab[chnId][0]
 *
 *       The associated quarter  : tab[chnId][1]  (0 means low, 1 means high quarter)
*/
static quartPlace mpod_equalzQuarters[12] = {
  {MPOD_TXEQUALZ1, 0},{MPOD_TXEQUALZ1, 1},{MPOD_TXEQUALZ2, 0},{MPOD_TXEQUALZ2, 1},{MPOD_TXEQUALZ3, 0},{MPOD_TXEQUALZ3, 1},
  {MPOD_TXEQUALZ4, 0},{MPOD_TXEQUALZ4, 1},{MPOD_TXEQUALZ5, 0},{MPOD_TXEQUALZ5, 1},{MPOD_TXEQUALZ6, 0},{MPOD_TXEQUALZ6, 1}
 };
/**
 *  \var mpod_ampControlCodes
 *  \brief Receiver output Amplitude - Default De-Amphasis given in mVppd unit
 *
 *     The they are 8 possible code values x = 0 to 7:
 *
 *     For each code we have the Min value tab[x][0], Nominal value tab[x][1], Max Value tab[x][2]
 *
 *     Note: another way to get the nominal value without this array is to compute (code+1)*100
*/
static unsigned mpod_ampControlCodes[][3] = {
	{ 70,100,130},
	{150,200,250},
	{240,300,360},
	{320,400,480},
	{400,500,600},/*default*/
	{480,600,720},
	{560,700,840},
	{640,800,960}/*full scale*/
	};
/**
 *  \var mpod_equalizationCodes
 *  \brief Transmitter equalization control
 *
 *     The they are 8 possible code values x = 0 to 7:
 *
 *     For each code we have the Peak value tab[x][0], Midband value tab[x][1], Peak vs Midband Value tab[x][2]
 *
*/
static float mpod_equalizationCodes[][3] = {
	{0.3 ,-0.1 ,0.4 },
	{1.3 ,-0.9 ,2.2 },
	{2.1 ,-1.8 ,3.9 },
	{3.05,-3.05,6.1 },
	{4.1 ,-4.7 ,8.8 },
	{5.0 ,-6.6 ,11.6},
	{5.6 ,-8.3 ,13.9},
	{6.1 ,-10.6,16.7}
	};
/* CHECK HOW THE READ WRITE BIT IS MANAGED IN THE DRIVER
# For TX the address is 0101hjkx (hjk is hard add) (x is 1=R 0=W)
# For RX the address is 0110hjkx (hjk is hard add) (x is 1=R 0=W)
# NOTE the x is added in Read/Write function into the driver so don't use it here
# partern 0 +   0101/TX or 0110/RX + fixAdd on 3digits

minipod devices devices addresses */
/**
 *  \var mpod_devAdd
 *  \brief The i2c addresses of the 8 channels controlers.
 *
*/
static unsigned mpod_devAdd[8] = { 
/* FR
  0x30,   //!<sRX GBT add(0) 0b 0011 0000  
  0x2a,   //!<sTX GBT add(1) 0b 0010 1010  
  0x31,   //!<sRX GBT add(2) 0b 0011 0001  
  0x2b,   //!<sTX GBT add(3) 0b 0010 1011  
  0x32,   //!<sRX GBT add(4) 0b 0011 0010  
  0x2c,   //!<sTX Gbe add(5) 0b 0010 1100  
  0x33,   //!<sRX Gbe add(6) 0b 0011 0011  
  0x2d,   //!<sTX Gbe add(7) 0b 0010 1101  
*/
/* MJ */ 
  0x30, //!<sRX GBT add(0) 0b 0011 0000 
  0x28, //!<sTX GBT add(1) 0b 0010 1000 
  0x31, //!<sRX GBT add(2) 0b 0011 0001 
  0x29, //!<sTX GBT add(3) 0b 0010 1001 
  0x32, //!<sRX GBT add(4) 0b 0011 0010 
  0x2a, //!<sTX Gbe add(5) 0b 0010 1010 
  0x33, //!<sRX Gbe add(6) 0b 0011 0011 
  0x2b, //!<sTX Gbe add(7) 0b 0010 1011 
};

static int mpod_rx[8] = {1, 0, 1, 0, 1, 0, 1, 0};
static int mpod_tx[8] = {0, 1, 0, 1, 0, 1, 0, 1};

static int mpod_scriptMode = 0; // mode of printing: 0 for human users else for script scanning
static int mpod_hideText = 0;

/***********************************************************************//**
 * \fn int mpod_setMode()
 * \brief Function to set the mode in which the data will be formatted on stdout.
 *       
 * By default the mode is 0 fir human typing the command in a system console.
 * Comments and pretty printings are used to make it easy for a human user 
 * to read the command results.
 * In the other mode the results are output in a minimalist comma separated values
 * which can be easily scanned by a scripting program.
 *
 * \param   mode: 0 human oriented, other for script scanning
 *//***********************************************************************/
 int mpod_setMode(int mode){
 	mpod_scriptMode=mode;
	return(0);
 }

 int mpod_setTextMode(int mode){
 	mpod_hideText=mode;
	return(0);
 }

/***********************************************************************//**
 * \fn int mpod_init(int dev)
 * \brief Function to init the avago i2c bus.
 *      This function has to be done before any i2c access to minipods
 *       
 * Function to program the prescale the SCL clock line. Should be called
 *        before the I2C use.
 *
 * \param   inCLK: the input system clock ferquency in Hz
 * \param SCLfreq: the target SCL clock frequency in Hz
 * \return 0 success, -1 error
 *//***********************************************************************/
int mpod_init(int dev, unsigned inCLK, unsigned SCLfreq){
    return( i2c_init(dev, I2C_BUS_MINIPODS, inCLK, SCLfreq) );
}
/***********************************************************************//**
 * \fn void mpod_getInit(int dev)
 * \brief Reads the last values in the prescale of the SCL clock line.
 *       
 *
 * \param   inCLK: the system clock ferquency in Hz
 * \param SCLfreq: the target SCL clock frequency in Hz
 *//***********************************************************************/
void mpod_getInit(int dev, unsigned *inCLK, unsigned *SCLfreq){
    i2c_getInit(dev, I2C_BUS_MINIPODS, inCLK, SCLfreq);
}
/**************************************************************************//**
 * \fn int mpod_exist(int mpid)
 * \brief Function to check if the given minipod exists
 *
 * \param mpid : the minipod number 0-7
 * \return 0 no, 1 yes
 *//**************************************************************************/
int mpod_exists(int mpid){
	if ((mpid<0) || (mpid>7)){
		printf("ERROR: minipod munber %d is not between 0 and 7\n",mpid);
		return(0);
	}
return 1;
}
/**************************************************************************//**
 * \fn int mpod_add(int mpid, unsigned *add)
 * \brief Function to get the I2C address of a given minipod
 *
 * \param mpid : the minipod number 0-7
 * \param add  : a pointer where to get the adress
 * \return 0 success, -1 error
 *//**************************************************************************/
static int mpod_add(int mpid, unsigned *add){
	if (!mpod_exists(mpid))
		return(-1);
	*add = mpod_devAdd[mpid];
return 0;
}
/**************************************************************************//**
 * \fn int mpod_isTX(int mpid)
 * \brief Function to test if a minipod is a transmitter
 *
 * \param mpid : the minipod number 0-7
 * \return 1=yes, 0=no
 *//**************************************************************************/
int mpod_isTX(int mpid){
	if (!mpod_exists(mpid))
		return(-1);
return(mpod_tx[mpid]);
}
/**************************************************************************//**
 * \fn int mpod_isRX(int mpid)
 * \brief Function to test if a minipod is a receiver
 *
 * \param mpid : the minipod number 0-7
 * \return 1=yes, 0=no
 *//**************************************************************************/
int mpod_isRX(int mpid){
	if (!mpod_exists(mpid))
		return(-1);
return(mpod_rx[mpid]);
}
/*********************************************************************//**
 * \fn int mpod_write(int dev, int mpid, char reg, unsigned val)
 * \brief Function to write val at register reg in minipod number mpid
 *
 * \param mpid: the target minipod
 * \param  reg: the device register
 * \param  val: the value to write (4bytes)
 * \return 0 success, -1 error
 *//*********************************************************************/
int mpod_write(int dev, int mpid, char reg, unsigned val){
	unsigned add;
	int ret;
	
	if(mpod_add(mpid, &add)!=0)
		return(-1);

	ret = i2c_writeMem(dev, I2C_BUS_MINIPODS, add, reg, &val);
	usleep(10000); // not clean but tested as usefull
	return (ret);
}
/******************************************************//**
 * \fn int mpod_read(int dev, int mpid, char reg, unsigned *val)
 * \brief Function to read register reg and get value in val
 *
 * \param mpid: the target minipod
 * \param  reg: the device register
 * \param  val: a ptr to a word to receive the value(4bytes)
 * \return 0 success, -1 error
 *//******************************************************/
int mpod_read(int dev, int mpid, char reg, unsigned *val){
	unsigned add;

	if(mpod_add(mpid, &add)!=0)
		return(-1);

	return( i2c_readMem(dev, I2C_BUS_MINIPODS, add, reg, val) );
}
/********************************************************************//**
 * \fn int mpod_readBloc(int dev, int file , int index , int nb, unsigned *data)
 * \brief Function to select the used memory page
 *
 * This function malloc the space to put the data in. The user should
 * free it after use.
 *
 * \param mpid : the target minipod number 0 to 7
 * \param index: the start register index
 * \param    nb: the number of registers to read
 * \param  data: a pointer to an array to receive the read values
 * \return 0 success, -1 error
 *//********************************************************************/
int mpod_readBloc(int dev, int mpid , int index , int nb , unsigned **data){
	unsigned add;

	if(mpod_add(mpid, &add)!=0)
		return(-1);
//	printf("PYD read on bus %d at add 0x%x reg index %d for %d words\n",I2C_BUS_Minipod, add, index, nb);
	return( i2c_multReadMem(dev, I2C_BUS_MINIPODS, add, index, nb, data) );
}
/******************************************************//**
 * \fn int mpod_pageSelect(int dev, int mpid, int page)
 * \brief Function to select the used memory page
 *
 * \param mpid: the target minipod
 * \param page: the selected page id (0 or 1)
 * \return 0 success, -1 error
 *//******************************************************/
int mpod_pageSelect(int dev, int mpid , unsigned page) {
	unsigned val;
	if ((page<0) || (page >1)){
		printf("ERROR: MPOD page number should be 0 or 1 not %d\n", page);
		return(-1);
	}
	// read the current value
//	printf("Setting page %d for minipod %d\n", page, mpid);
	mpod_write(dev, mpid, MPOD_PAGEREG, page);
	mpod_read(dev,  mpid, MPOD_PAGEREG, &val);
	if (val!=page){
		printf("ERROR: page selection on mpod %d failed\n", mpid);
		return(-1);
	}
//	printf("Current page is now %d for minipod %d\n",val, mpid);
	return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_get12IntData(int(*get)(int,int**), int mpid, int *data)
 * \brief Function to wrap other functions to get the 12 channels data of a minipod in python
 *
 * Function to avoid to have to manage dynamically allocated arrays in the python wrapper.
 * used for getVOD, getDeamphasis, getEqualization
 *
 * \param mpid: the target RX minipod component 0, 2, 4 and 6 (RX)
 * \param data: a pointer to an already allocated array of 12 code values
 *              (chn 0 to 11) that the user will have to freed after use.
 * \return 0 success, -1 error
 *//****************************************************************************************/
static int mpod_get12IntData(int dev,int(*get)(int,int,int**),int mpid, int *data){
	int *bloc;
	int i, status;
	status = get(dev,mpid, &bloc);
	if (status==0){
	
		for (i=0; i<12; i++)
			data[i]=bloc[i];
		free(bloc);
	}
	return(status);
}
/*****************************************************************************************//**
 * \fn int mpod_get12FloatData(int(*get)(int,float**), int mpid, float *data)
 * \brief Function to wrap other functions to get the 12 channels data of a minipod in python
 *
 * Function to avoid to have to manage dynamically allocated arrays in the python wrapper.
 * used for biasCurrent, lightOutput, lightInput
 *
 * \param mpid: the target RX minipod component 0, 2, 4 and 6 (RX)
 * \param data: a pointer to an already allocated array of 12 code values
 *              (chn 0 to 11) that the user will have to freed after use.
 * \return 0 success, -1 error
 *//****************************************************************************************/
static int mpod_get12FloatData(int dev, int(*get)(int,int,float**),int mpid, float *data){
	float *bloc;
	int i, status;
	status = get(dev, mpid, &bloc);
	if (status==0){
		for (i=0; i<12; i++)
			data[i]=bloc[i];
		free(bloc);
	}
	return(status);
}
/*****************************************************************************************//**
 * \fn int mpod_setBitIn12bitsOn2words (int dev, int mpid, int channel, int flag
 													unsigned word1, unsigned word2,
 													char *regMsg,
													(int)(*filter)(int))
 * \brief Function to set or unset a bit for a channel in a representation spanning on
 *        2 contiguous 8 bits registers.
 *
 * \param mpid   : the target minipod component 0 to 5
 * \param channel: the target channel 0 to 12
 * \param flag   : 0=enable  1=disable
 * \param word1  : low number register with bits for channels 8 to 11
 * \param word2  : high number register with bits for channels 0 to 7
 * \param regMsg : string identifying the register function
 * \param filter : filtering function for the type of minipod
 * \return 0 success, -1 error
 *//****************************************************************************************/
static int mpod_setBitIn12bitsOn2words(int dev, int mpid, int channel, int flag, 
											unsigned word1, unsigned word2,
											char *regMsg,
											int(*filter)(int) ){
int shift;
unsigned reg, cval, val;

if (filter)
 if (!filter(mpid)){
 	printf("ERROR: %s only for TX minipods\n", regMsg);
 	return(-1);
 }
 if (channel<8){
	reg = word2;
	shift = channel;
	}
 else if (channel<=12) {
	reg = word1;
	shift = channel - 8;
	}
 else {
	printf("ERROR : channel should be in <0-11>\n");
	return(-1);
 }
 mpod_read(dev, mpid, reg, &cval);
 if (flag){ //disable
	val = cval | (1<<shift);
	if (mpod_write(dev, mpid, reg, val)){
		printf("ERROR: can't %s for channel %d in minipod %d board %d\n", regMsg, channel, mpid, dev);
		return(-1);
  }
 } 
 else { //enable
	val = cval & ~(1<<shift);
	if (mpod_write(dev, mpid, reg, val)){
		printf("ERROR: can't %s for channel %d in minipod %d board %d\n", regMsg, channel, mpid, dev);
		return(-1);
	}
 }
return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_dumpAllBitsIn12bitsOn2words (int dev, int mpid, 
 													unsigned word1, unsigned word2,
 													char *regMsg,
													(int)(*filter)(int))
 * \brief Function to dump all values coded as 1 bits per channel in 2 contiguous 8 bits registers.
 *
 * \param mpid   : the target minipod component 0 to 7
 * \param word1  : low number register with bits for channels 8 to 11
 * \param word2  : high number register with bits for channels 0 to 7
 * \param regName: string identifying the register function
 * \param filter : filtering function for the type of minipod
 * \param no     : character for 0
 * \param yes    : character for 1
 * \return 0 success, -1 error
 *//****************************************************************************************/
static int mpod_dumpAllBitIn12bitsOn2words(int dev, int mpid, 
											unsigned word1, unsigned word2,
											char *regName,
											int(*filter)(int),
											char no, char yes ){
	unsigned val;
	int ch;
	
	if (filter)
	 if (filter(mpid)){
		printf("ERROR: %s registers not available for this type of minipod\n", regName);
		return(0);
	}
	if (!mpod_scriptMode){
		for (ch=0; ch<12; ch++) printf("---");
			printf("\n");
		printf("minipod %d %s\n", mpid, regName);
		for (ch=0; ch<12; ch++) printf("---");
		printf("\n");
		for (ch=0; ch<12; ch++) printf("%2d ",ch);
		printf("\n");	
		for (ch=0; ch<12; ch++) printf("---");
		printf("\n");
	}
	if (mpod_read(dev, mpid, word2, &val )){
		printf("ERROR: MPOD can't read %s register 2 on i2c bus\n", regName);
		return(-1);
	}
	if (!mpod_scriptMode){
		for (ch=0; ch<8; ch++){
			if ( (val & (1<<ch)) != 0 ) printf(" %c ",yes);
			else                        printf(" %c ",no);
		}
	} else {
		for (ch=0; ch<8; ch++){
			if ( (val & (1<<ch)) != 0 ) printf("%c,",yes);
			else                        printf("%c,",no);
		}	
	}
	if (mpod_read(dev, mpid, word1, &val )){
		printf("ERROR: MPOD can't read %s register 1 on i2c bus\n", regName);
		return(-1);
	}
	if (!mpod_scriptMode){
		for (ch=8; ch<12; ch++){
			if ( (val & (1<<(ch-8))) != 0 ) printf(" %c ",yes);
			else                            printf(" %c ",no);
		}
	} else {
		for (ch=8; ch<12; ch++){
			if ( (val & (1<<(ch-8))) != 0 ) ch==11 ? printf("%c",yes): printf("%c,",yes);
			else                            ch==11 ? printf("%c",no): printf("%c,",no);
		}	
	}
	printf("\n");
	if (!mpod_scriptMode){
		for (ch=0; ch<12; ch++) printf("---");
			printf("\n");
	}
	return(0);
}

/*****************************************************************************************//**
 * \fn int mpod_mkListBitsIn12bitsOn2words (int dev, int mpid, 
 													unsigned word1, unsigned word2,
                                       char *regName,
													(int)(*filter)(int),
                                       char no,
                                       char yes,
                                       char **data)
 * \brief Function to list all values coded as 1 bits per channel in 2 contiguous 8 bits registers.
 *
 * The first register has 8 bits and the second 4 bits.
 *
 * This function is used by the python wrapper
 *
 * \param mpid   : the target minipod component 0 to 7
 * \param word1  : low number register with bits for channels 8 to 11
 * \param word2  : high number register with bits for channels 0 to 7
 * \param filter : filtering function for the type of minipod
 * \param no     : character for 0
 * \param yes    : character for 1
 * \param data   : the comma separated values in a string allocated by this function that should be freed
 * \return 0 success, -1 error
 *//****************************************************************************************/
static int mpod_mkListBitIn12bitsOn2words(int dev, int mpid, 
                                          unsigned word1, unsigned word2,
                                          char *regName,
                                          int(*filter)(int),
                                          char no, char yes,
                                          char **data ){
	unsigned val;
	int ch;	
        char *str;
	
	// Allocate the memory to make a string
	if (!(str = (char *)malloc(50))) { // 12 value with , except last followed by \n
		printf("ERROR: Can't malloc memory for 12 comma separated values\n");
		return(-1);
	}
	for  (ch=0; ch<50; ch++)
	    str[ch] = '\n';
	    
	if (filter)
	 if (filter(mpid)){
		printf("ERROR: %s registers not available for this type of minipod\n", regName);
		return(0);
	}
	if (mpod_read(dev, mpid, word2, &val )){
		printf("ERROR: MPOD can't read %s register 2 on i2c bus\n", regName);
		return(-1);
	}
	for (ch=0; ch<8; ch++){
		if ( (val & (1<<ch)) != 0 ) {
		    sprintf(str+2*ch,"%c,",yes);
		}
		else{
		    sprintf(str+2*ch,"%c,",no);
		}
	}	
	if (mpod_read(dev, mpid, word1, &val )){
		printf("ERROR: MPOD can't read %s register 1 on i2c bus\n", regName);
		return(-1);
	}
	for (ch=8; ch<12; ch++){
		if ( (val & (1<<(ch-8))) != 0 ) {
		    if (ch == 11){
		        sprintf(str+2*ch,"%c",yes);
		    }
		    else {
		        sprintf(str+2*ch,"%c,",yes);
		        }      
		}
		else {
		    if (ch == 11){
		        sprintf(str+2*ch,"%c",no);
		        }
		        
		    else {
		        sprintf(str+2*ch,"%c,",no);
		        }
		}
	}
	*data = str;
	return(0);
}
/********************************************************************//**
 * \fn int mpod_getVendorName(int dev, int mpid , char **bloc)
 * \brief Function read the vendor name from register 152 to 167
 *
 * \param mpid : the target minipod component 0 to 7
 * \param bloc : a pointer to an array to receive the read values
 *         The bloc is allocated by the function and should be freed
 *         by the user
 * \return 0 success, -1 error
 *//********************************************************************/
int mpod_getVendorName(int dev, int mpid, char **bloc){
	int i;
	char     *chdata; // allocated here tp make conversion from unsigned to char
	unsigned *data;   // will be allocated by the called function
	int size =   MPOD_NBVENDORNAME+1;

	// Allocate the memory to make a string
	if (!(chdata = (char *)malloc(size+1))) {
		printf("ERROR: Can't malloc memory for vendor data\n");
		return(-1);
	}
	// get data
	if ( mpod_readBloc(dev, mpid , MPOD_VENDORNAMEREG , size , &data )!=0){
		printf("ERROR: MPOD can't read vendor name on i2c bus\n");
		return(-1);
	}

	for (i=0; i<MPOD_NBVENDORNAME; i++){
		chdata[i] = (char)data[i];
	}

	chdata[i]='\0';	
	free(data);
	*bloc = chdata;
	return(0);
}
/********************************************************************//**
 * \fn int mpod_getVendorOUI(int dev, int mpid , char **bloc)
 * \brief Function read the vendor OUIom register 168 to 170
 *
 * Attention: the characters returned are not printable but values
 * to print as hexa digits to be interpreted.
 *
 * \param mpid : the target minipod component 0 to 7
 * \param bloc : a pointer to an array to receive the read values
 *         The bloc is allocated by the function and should be freed
 *         by the user
 * \return 0 success, -1 error
 *//********************************************************************/
int mpod_getVendorOUI(int dev, int mpid, char **bloc){
	int i;
	char     *chdata; // allocated here tp make conversion from unsigned to char
	unsigned *data;   // will be allocated by the called function
	int size =   MPOD_NBVENDOROUI+1;

	// Allocate the memory to make a string
	if (!(chdata = (char *)malloc(size+1))) {
		printf("ERROR: Can't malloc memory for vendor OUI data\n");
		return(-1);
	}
	// get data
	if ( mpod_readBloc(dev, mpid , MPOD_VENDOROUIREG , size , &data )!=0){
		printf("ERROR: MPOD can't read vendor OUI on i2c bus\n");
		return(-1);
	}
	for (i=0; i<MPOD_NBVENDOROUI; i++){
		chdata[i] = (char)data[i];
	}
	free(data);
	*bloc = chdata;
	return(0);
}
/********************************************************************//**
 * \fn int mpod_getVendorPartNumber(int dev, int mpid , char **bloc)
 * \brief Function read the vendor part number from register 171 to 186
 *
 * \param mpid : the target minipod component 0 to 7
 * \param bloc : a pointer to an array to receive the read values
 *         The bloc is allocated by the function and should be freed
 *         by the user
 * \return 0 success, -1 error
 *//********************************************************************/
int mpod_getVendorPartNumber(int dev, int mpid, char **bloc){
	int i;
	char     *chdata; // allocated here tp make conversion from unsigned to char
	unsigned *data;   // will be allocated by the called function
	int size =   MPOD_NBVENDORPART+1;

	// Allocate the memory to make a string
	if (!(chdata = (char *)malloc(size+1))) {
		printf("ERROR: Can't malloc memory for vendor part number data\n");
		return(-1);
	}
	// get data
	if ( mpod_readBloc(dev, mpid , MPOD_VENDORPARTREG , size , &data )!=0){
		printf("ERROR: MPOD can't read vendor part number on i2c bus\n");
		return(-1);
	}
	for (i=0; i<MPOD_NBVENDORPART; i++){
		chdata[i] = (char)data[i];
	}
	chdata[i]='\0';
	free(data);
	*bloc = chdata;
	return(0);
}
/********************************************************************//**
 * \fn int mpod_getVendorRevNumber(int dev, int mpid , char **bloc)
 * \brief Function read the revision number from register 187 to 188
 *
 * \param mpid : the target minipod component 0 to 7
 * \param bloc : a pointer to an array to receive the read values
 *         The bloc is allocated by the function and should be freed
 *         by the user
 * \return 0 success, -1 error
 *//********************************************************************/
int mpod_getVendorRevNumber(int dev, int mpid, char **bloc){
	int i;
	char     *chdata; // allocated here tp make conversion from unsigned to char
	unsigned *data;   // will be allocated by the called function
	int size =   MPOD_NBVENDORREV+1;

	// Allocate the memory to make a string
	if (!(chdata = (char *)malloc(size+1))) {
		printf("ERROR: Can't malloc memory for vendor revision number data\n");
		return(-1);
	}
	// get data
	if ( mpod_readBloc(dev, mpid , MPOD_VENDORREVREG , size , &data )!=0){
		printf("ERROR: MPOD can't read vendor revision number on i2c bus\n");
		return(-1);
	}
	for (i=0; i<MPOD_NBVENDORREV; i++){
		chdata[i] = (char)data[i];
	}
	chdata[i]='\0';
	free(data);
	*bloc = chdata;
	return(0);
}
/********************************************************************//**
 * \fn int mpod_getVendorSerialNumber(int dev, int mpid , char **bloc)
 * \brief Function read the vendor serial number from register 189 to 204
 *
 * \param mpid : the target minipod component 0 to 7
 * \param bloc : a pointer to an array to receive the read values
 *         The bloc is allocated by the function and should be freed
 *         by the user
 * \return 0 success, -1 error
 *//********************************************************************/
int mpod_getVendorSerialNumber(int dev, int mpid, char **bloc){
	int i;
	char     *chdata; // allocated here tp make conversion from unsigned to char
	unsigned *data;   // will be allocated by the called function
	int size =   MPOD_NBVENDORSER+1;

	// Allocate the memory to make a string
	if (!(chdata = (char *)malloc(size+1))) {
		printf("ERROR: Can't malloc memory for vendor serial number data\n");
		return(-1);
	}
	// get data
	if ( mpod_readBloc(dev, mpid , MPOD_VENDORSERREG , size , &data )!=0){
		printf("ERROR: MPOD can't read vendor serial number on i2c bus\n");
		return(-1);
	}
	for (i=0; i<MPOD_NBVENDORSER; i++){
		chdata[i] = (char)data[i];
	}
	chdata[i]='\0';
	free(data);
	*bloc = chdata;
	return(0);
}
/********************************************************************//**
 * \fn int mpod_getVendorDate(int dev, int mpid , char **bloc)
 * \brief Function read the vendor date from register 205 to 212
 *
 * \param mpid : the target minipod component 0 to 7
 * \param bloc : a pointer to an array to receive the read values
 *         The bloc is allocated by the function and should be freed
 *         by the user
 * \return 0 success, -1 error
 *//********************************************************************/
int mpod_getVendorDate(int dev, int mpid, char **bloc){
	int i;
	char     *chdata; // allocated here tp make conversion from unsigned to char
	unsigned *data;   // will be allocated by the called function
	int size =   MPOD_NBVENDORDATE+1;

	// Allocate the memory to make a string
	if (!(chdata = (char *)malloc(size+1))) {
		printf("ERROR: Can't malloc memory for vendor date data\n");
		return(-1);
	}
	// get data
	if ( mpod_readBloc(dev, mpid , MPOD_VENDORDATEREG , size , &data )!=0){
		printf("ERROR: MPOD can't read vendor date on i2c bus\n");
		return(-1);
	}
	for (i=0; i<MPOD_NBVENDORDATE; i++){
		chdata[i] = (char)data[i];
	}
	chdata[i]='\0';
	free(data);
	*bloc = chdata;
	return(0);
}
/********************************************************************//**
 * \fn int mpod_dumpVendorData(int dev, int mpid)
 * \brief Function print the vendor data of a minipod
 *
 * \param mpid : the target minipod component 0 to 7
 * \return 0 success, -1 error
 *//********************************************************************/
int mpod_dumpVendorData(int dev, int mpid){
	int i;
	char *bloc;
	
	if (!mpod_scriptMode){
		printf("----------------------\n");
		printf("minipod %d vendor data\n",  mpid);
		printf("----------------------\n");
	};
	if (mpod_getVendorName(dev, mpid, &bloc)==-1) return(-1);
	else {
		printf("vendor name     :%s\n", bloc);
		free(bloc);
	}
	if (mpod_getVendorOUI(dev, mpid, &bloc)==-1) return(-1);
	else {
		printf("vendor OUI      :");
		for (i=0; i<MPOD_NBVENDOROUI; i++)
			printf("0x%02xh ",bloc[i]);
		printf("\n");
		free(bloc);
	}
	if (mpod_getVendorPartNumber(dev, mpid, &bloc)==-1) return(-1);
	else {
		printf("part number     :%s\n", bloc);
		free(bloc);
	}
	if (mpod_getVendorRevNumber(dev, mpid, &bloc)==-1) return(-1);
	else {
		printf("revision number :%s\n", bloc);
		free(bloc);
	}
	if (mpod_getVendorSerialNumber(dev, mpid, &bloc)==-1) return(-1);
	else {
		printf("serial number   :%s\n", bloc);
		free(bloc);
	}
	if (mpod_getVendorDate(dev, mpid, &bloc)==-1) return(-1);
	else {
		printf("date            :%s\n", bloc);
		free(bloc);
	}
	return(0);
}
/********************************************************************//**
 * \fn int mpod_getVendorData(int dev, int mpid)
 * \brief Function to get a string with the vendor data of a minipod
 *
 * Function used in the python wrapper
 *
 * \param mpid : the target minipod component 0 to 7
 * \param data : the vendor data's string allocated here that should be
 * \param        by the caller              
 * \return 0 success, -1 error
 *//********************************************************************/
int mpod_getVendorData(int dev, int mpid, char **data){
	int i;
	char *bloc;
	char *str;
	
	if ((str = (char *)malloc(1000))==0){
		printf("ERROR: Can't malloc memory to build the vendor data info string\n");
		return(-1);
	}
	sprintf(str+strlen(str),"minipod:%d,", mpid);
	if (mpod_getVendorName(dev, mpid, &bloc)==-1) return(-1);
	else {
		sprintf(str+strlen(str),"vendor-name:%s,", bloc);
		free(bloc);
	}
	if (mpod_getVendorOUI(dev, mpid, &bloc)==-1) return(-1);
	else {
		sprintf(str+strlen(str),"vendor-OUI:");
		for (i=0; i<MPOD_NBVENDOROUI; i++)
			sprintf(str+strlen(str),"0x%02xh ",bloc[i]);
		sprintf(str+strlen(str),",");
		free(bloc);
	}
	if (mpod_getVendorPartNumber(dev, mpid, &bloc)==-1) return(-1);
	else {
		sprintf(str+strlen(str),"part-number:%s,", bloc);
		free(bloc);
	}
	if (mpod_getVendorRevNumber(dev, mpid, &bloc)==-1) return(-1);
	else {
		sprintf(str+strlen(str),str+strlen(str),"revision-number:%s,", bloc);
		free(bloc);
	}
	if (mpod_getVendorSerialNumber(dev, mpid, &bloc)==-1) return(-1);
	else {
		sprintf(str+strlen(str),"serial-number:%s,", bloc);
		free(bloc);
	}
	if (mpod_getVendorDate(dev, mpid, &bloc)==-1) return(-1);
	else {
		sprintf(str+strlen(str),"date:%s", bloc);
		free(bloc);
	}
	*data = str;
	return(0);
}
/*********************************************************************************//**
 * \fn int mpod_squelchDisable(int dev, int mpid , int channel , int disable)
 * \brief Function to disable/enable squelch on one channel of a component
 *
 * \param mpid   : the target minipod component 0 to 7
 * \param channel: the channel number 0-11
 * \param disable: the flag 1=disable 0=enable
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_squelchDisable(int dev, int mpid , int channel , int disable){

char regMsg[]="squelch disable";
return (mpod_setBitIn12bitsOn2words(dev, mpid, 
												channel, 
												disable, 
												MPOD_SQUELCHREG1, 
												MPOD_SQUELCHREG2,
												regMsg, 
												NULL));
}
/*********************************************************************************//**
 * \fn int mpod_squelchDisableAll(int dev, int mpid , int disable)
 * \brief Function to disable/enable squelch on all channels of a component
 *
 * \param mpid   : the target minipod component 0 to 7
 * \param disable: the flag 1=disable 0=enable
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_squelchDisableAll(int dev, int mpid, int disable){
	int mask;
	if (disable){
		printf("Switching squelch to OFF on ALL channels of component %d\n", mpid);
		mask = 0xff;
		if (mpod_write(dev, mpid, MPOD_SQUELCHREG2, mask)){
			printf("ERROR: MPOD can't write squelch register on i2c bus\n");
			return(-1);
		};
		mpod_write(dev, mpid, MPOD_SQUELCHREG1, mask); /*also write useless high quarter doesn't harm*/
	} else {
		printf("Switching squelch to ON on ALL channels of component %d\n", mpid);
		mask = 0x0;
		if (mpod_write(dev, mpid, MPOD_SQUELCHREG2, mask)){
			printf("ERROR: MPOD can't write squelch register on i2c bus\n");
			return(-1);
		};
		mpod_write(dev, mpid, MPOD_SQUELCHREG1, mask); /*also write useless high quarter doesn't harm*/
	}
	return(0);
}
/*********************************************************************************//**
 * \fn int mpod_squelchDumpAll(int dev, int mpid)
 * \brief Function to dump the value of squelch enabled status for all channels
 *
 * \param mpid: the target minipod component 0 to 7
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_squelchDumpAll(int dev, int mpid){
char regName[]="Squelch enable/disable";

return mpod_dumpAllBitIn12bitsOn2words(dev, mpid, 
											MPOD_SQUELCHREG1, MPOD_SQUELCHREG2,
											regName,
											NULL,
											'e','d');
}
/*********************************************************************************//**
 * \fn int mpod_dumpErrStatusValues(int dev, int mpid)
 * \brief Function to dump the data not ready, int, los and faul  status of one minipod
 *
 * \param mpid: the target minipod component 0 to 7
 * \return 0 success, -1 error
 *//*********************************************************************************/
static int mpod_dumpErrStatusValues(int dev, int mpid){
unsigned val;
	if (mpod_read(dev, mpid, MPOD_ERRORSTAT, &val )){
		printf("ERROR: MPOD can't read error status register on i2c bus\n");
		return(-1);
	}
	if (!mpod_scriptMode){
		if (mpod_isTX(mpid))
			printf("!  %d   !  %d    !  %d  !  %d  !        %d       !\n", mpid, 3>(val&0x8), 2>(val&0x4), 1>(val&0x2), val&01);
		else
			printf("!  %d   !  *    !  %d  !  %d  !        %d       !\n",  mpid,              2>(val&0x4), 1>(val&0x2), val&01);
	} else {
		if (mpod_isTX(mpid))
			printf("%d,%d,%d,%d,%d\n", mpid, 3>(val&0x8), 2>(val&0x4), 1>(val&0x2), val&01);
		else
			printf("%d,*,%d,%d,%d\n",  mpid,              2>(val&0x4), 1>(val&0x2), val&01);		
	}
	
	return 0;
}
/*********************************************************************************//**
 * \fn int mpod_getErrStatus(int dev, int mpid, char **data)
 * \brief Function to get the data not ready,int,los and faul status of one minipod
 *
 * Function used by the python wrapper
 * \param mpid: the target minipod component 0 to 7
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_getErrStatus(int dev, int mpid, char **data){
unsigned val;
char *str;
	if ((str = (char *)malloc(100))==0){
		printf("ERROR: Can't malloc memory to build the error status info string\n");
		return(-1);
	}
	if (mpod_read(dev, mpid, MPOD_ERRORSTAT, &val )){
		printf("ERROR: MPOD can't read error status register on i2c bus\n");
		return(-1);
	}
	if (mpod_isTX(mpid))
		sprintf(str,"fault:%d,los:%d,intL:%d,data-not-ready:%d\n", 3>(val&0x8), 2>(val&0x4), 1>(val&0x2), val&01);
	else
		sprintf(str,"fault:*,los:%d,intL:%d,data-not-ready:%d\n",                2>(val&0x4), 1>(val&0x2), val&01);		
   *data = str;
	return(0);
}
/*********************************************************************************//**
 * \fn int mpod_dumpErrStatus(int dev, int mpid)
 * \brief Function to dump the data not ready, int, los and faul  status of one minipod
 *
 * \param mpid: the target minipod component 0 to 7
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_dumpErrStatus(int dev, int mpid){
	if (!mpod_scriptMode){
		printf("minipod %d error status\n", mpid);
		printf("!-------------------------------------------!\n");
		printf("! MPOD ! FAULT ! LOS ! INT ! DATA NOT READY !\n");
		printf("!-------------------------------------------!\n");
	}
	mpod_dumpErrStatusValues(dev, mpid);
	if (!mpod_scriptMode)
		printf("!-------------------------------------------!\n");
return(0);
}
/*********************************************************************************//**
 * \fn int mpod_dumpErrStatusAll(int dev)
 * \brief Function to dump the data not ready, int, los and faul  status of all minipods
 *
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_dumpErrStatusAll(int dev){
int     mpid;
	if (!mpod_scriptMode){
		printf("minipod all error status\n");
		printf("!-------------------------------------------!\n");
		printf("! MPOD ! FAULT ! LOS ! INT ! DATA NOT READY !\n");
		printf("!-------------------------------------------!\n");
	}
	for (mpid=0; mpid<=7; mpid++){
		mpod_dumpErrStatusValues(dev, mpid);
	}
	if (!mpod_scriptMode)
		printf("!-------------------------------------------!\n");
return(0);
}
/*********************************************************************************//**
 * \fn int mpod_losDumpAll(int dev, int mpid)
 * \brief Function to dump the los status for all channels of one minipod
 *
 * \param mpid: the target minipod component 0 to 7
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_losDumpAll(int dev, int mpid){
char regName[]="Loss of signal";

return mpod_dumpAllBitIn12bitsOn2words(dev, mpid, 
											MPOD_LOSREG1, MPOD_LOSREG2,
											regName,
											NULL,
											'0','1');
}
/*********************************************************************************//**
 * \fn int mpod_losListAll(int dev, int mpid, char **data)
 * \brief Function to make a list of the the los status for all channels of one minipod
 *
 * Attention bits should be reordered for 0 to 11
 * Function used by the python wrapper
 *
 * \param mpid: the target minipod component 0 to 7
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_losListAll(int dev, int mpid, char **data){
char regName[]="Loss of signal";
int status;

// attention clear on read register
status = mpod_mkListBitIn12bitsOn2words(dev, mpid, 
				MPOD_LOSREG1, MPOD_LOSREG2,
				regName,
				NULL,
				'0','1',
                                 data );
printf("Loss of signal :%s\n", *data);

return status;
}
/*********************************************************************************//**
 * \fn int mpod_fualtDumpAll(int dev, int mpid)
 * \brief Function to dump the fault status for all channels of one TX  minipod
 *
 * \param mpid: the target minipod component 0 to 7
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_faultDumpAll(int dev, int mpid){
char regName[]="Faults";

return mpod_dumpAllBitIn12bitsOn2words(dev, mpid, 
											MPOD_FAULTREG1, MPOD_FAULTREG2,
											regName,
											mpod_isRX,
											'0','1');
}
/*********************************************************************************//**
 * \fn int mpod_internalTemp(int dev, int mpid, int *temp)
 * \brief Function to get the temperature value of one minipod
 *
 * \param mpid: the target minipod component 0 to 7
 * \param temp: a pointer to the variable to receive the temperature
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_internalTemp(int dev, int mpid, int *temp){
	unsigned valh;

	// only use the higher temp at +- 3°C
	if (mpod_read(dev, mpid , MPOD_INTEMPMSB , &valh )){
		printf("ERROR: MPOD can't read the internal temperature register on i2c bus\n");
		return(-1);
	}
	*temp = valh;
	if (mpod_hideText);
	else if (mpod_scriptMode)
		printf("%d,",*temp);
	else
		printf("minipod %d internal temperature %dC\n",mpid, *temp);
		
//	printf("Internal temperature of minipod %d is %d°C\n", mpid, *temp);
	return(0);
}
/*********************************************************************************//**
 * \fn int mpod_internalTempDumpAll(int dev)
 * \brief Function to print the temperatures value of all minipods
 *
 * \param mpid: the target minipod component 0 to 7
 * \param temp: a pointer to the variable to receive the temperature
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_internalTempDumpAll(int dev){
	int i, temp;
	for (i=0; i<8; i++){
		mpod_internalTemp(dev, i, &temp);
	}
	return(0);
}
/*********************************************************************************//**
 * \fn int mpod_internal33Vcc(int dev, int mpid, int *vcc)
 * \brief Function to get the 3.3 vCC value of one minipod
 *
 * \param mpid: the target minipod component 0 to 7
 * \param  vcc: a pointer to the variable to receive the voltage in uV
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_internal33Vcc(int dev, int mpid, int *vcc){
	unsigned valh, vall;
	// see how the target device is specified ... compute target device address
	if (mpod_read(dev, mpid, MPOD_33VCCMSB , &valh )){
		printf("ERROR: MPOD can't read the 3.3Vcc register on i2c bus\n");
		return(-1);
	}
	mpod_read(dev, mpid , MPOD_33VCCLSB , &vall);
	*vcc = (valh<<8) + vall; // value in uV
   if (mpod_hideText);
	else if (mpod_scriptMode)
		printf("%.2f,",(float)*vcc/10000);
	else
		printf("3.3 power of minipod %d is %.2fV\n", mpid, (float)*vcc/10000);
	return(0);
}
/*********************************************************************************//**
 * \fn int mpod_internal33VccDumpAll(int dev)
 * \brief Function to get the 3.3 vCC value of all minipods voltage in uV
 *
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_internal33VccDumpAll(int dev){
	int i, temp;
	for (i=0; i<8; i++){
		mpod_internal33Vcc(dev, i, &temp);
	}
	return(0);
}
/*********************************************************************************//**
 * \fn int mpod_internal25Vcc(int dev, int mpid, int *vcc)
 * \brief Function to get the 2.5 vCC value of one minipod
 *
 * \param mpid: the target minipod component 0 to 7
 * \param  vcc: a pointer to the variable to receive the voltage in uV
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_internal25Vcc(int dev, int mpid, int *vcc){
	unsigned valh, vall;
	// see how the target device is specified ... compute target device address
	if (mpod_read(dev, mpid, MPOD_25VCCMSB , &valh )){
		printf("ERROR: MPOD can't read the 2.5Vcc register on i2c bus\n");
		return(-1);
	}
	mpod_read(dev, mpid , MPOD_25VCCLSB , &vall);
	*vcc = (valh<<8) + vall;
	if (mpod_hideText);
	else if (mpod_scriptMode)
		printf("%2.2f,",(float)*vcc/10000);
	else
		printf("2.5 power of minipod %d is %2.2fV\n", mpid, (float)*vcc/10000);
	return(0);
}
/*********************************************************************************//**
 * \fn int mpod_internal25VccDumpAll(int dev)
 * \brief Function to get the 2.5 vCC value of all minipods voltage in uV
 *
 * \return 0 success, -1 error
 *//*********************************************************************************/
int mpod_internal25VccDumpAll(int dev){
	int i, temp;
	for (i=0; i<8; i++){
		mpod_internal25Vcc(dev, i, &temp);
	}
	return(0);
}
/***********************************************************************************//**
 * \fn int mpod_biasCurrent(int dev, int mpid, float **data)
 * \brief Function to get and print all bias current values of one TX minipod
 *
 * \param mpid: the target minipod component 1, 3, 5 ,7
 * \param data: a pointer at which the function will put an allocated array of 12 values
 *              (chn 0 to 11) that the user will have to freed after use.
 *              value returned in the array are in uA
 * \return 0 success, -1 error
 *//***********************************************************************************/
int mpod_biasCurrent(int dev, int mpid, float **data){
	unsigned valh, vall, reg, chn;
	float val;
	float *cval;
	
	if (!(mpod_isTX(mpid))){
		printf("ERROR: bias current monitoring only available for TX minipods\n");
		return(-1);
	};
	if (!mpod_scriptMode)
		printf("Bias current of minipod %d\n", mpid);
	if (!(cval = (float *)malloc(12*sizeof(float)))){
		printf("ERROR: Can't malloc memory for bias current data\n");
		return(-1);
	}
	/* All the  registers for all channels are in sequence starting with channel 11 downto 0.
	   They are coded on two bytes the first beeing the MSB. */
	reg = MPOD_BIASCURLSB00;
	for (chn=0; chn<12; chn++){
		if (mpod_read(dev, mpid, reg , &vall )){
			printf("ERROR: MPOD can't read the bias current register on i2c bus\n");
			return(-1);
		}
		mpod_read(dev, mpid, reg-1 , &valh);
		val = (float)((valh<<8) + vall) * 2; // unit is 2 uA
		if (!mpod_scriptMode)
			printf("[%2d] %4.1f uA\n", chn, val);
		else
			printf("%4.1f,", val);
		reg=reg-2;
	}//for
	if (mpod_scriptMode) printf("\n"); // change line for each minipod
	*data = cval;
	return(0);
}
/***********************************************************************************//**
 * \fn int mpod_getBiasCurrent(int dev, int mpid, float *data)
 * \brief Function to get and print all bias current values of one TX minipod in python
 *
 * \param mpid: the target minipod component 1, 3, 5 ,7
 * \param data: a pointer to an already allocated array of 12 values
 *              
 * \return 0 success, -1 error
 *//***********************************************************************************/
int mpod_getBiasCurrent(int dev, int mpid, float *data){
	return mpod_get12FloatData(dev, mpod_biasCurrent, mpid, data);
}
/***********************************************************************************//**
 * \fn int mpod_dumpLightPower(	int dev, int mpid,
 	int reg,
 	char *regMsg,
 	(int)(*filter)(int),
 	float **data)
 * \brief Function to get and print all light power values organized as 2 bytes MSB/LSB
 *        per parameter. values are in 0.1 uW units coded 16 bits unsigned integers
 *
 * \param mpid		: the target minipod component 0, 2 or 4
 * \param reg 		: the LSB register number for module 0
 * \param regMsg	: the register identification message
 * \param filter  : the filtering function to select proper type of minipods
 * \param data		: a pointer at which the function will put an allocated array of 11 values
 *              	  (chn 0 to 11) that the user will have to freed after use.
 *              	  value returned in the array are in uW
 * \return 0 success, -1 error
 *//***********************************************************************************/
static int mpod_dumpLightpower(int dev, int mpid,int reg,char *regMsg,int (*filter)(int),float **data){
	unsigned valh, vall, chn;
	float val;
	float *cval;
	
	if (filter)
 	  if (!filter(mpid)){
 		printf("ERROR: %s doesn't apply to this type of minipod\n", regMsg);
 		return(-1);
 	}
	if (!mpod_scriptMode)
		printf("%s of minipod %d\n", regMsg, mpid);
	if (!(cval = (float *)malloc(12*sizeof(float)))){
		printf("ERROR: Can't malloc memory for %s data\n", regMsg);
		return(-1);
	}
	/* All the  registers for all channels are in sequence starting with channel 11 downto 0.
	   They are coded on two bytes the first beeing the MSB. */
	for (chn=0; chn<12; chn++){
		if (mpod_read(dev, mpid, reg , &vall )){
			printf("ERROR: MPOD can't read the %s register on i2c bus\n",regMsg);
			return(-1);
		}
		mpod_read(dev, mpid, reg-1 , &valh);
		val = (float)((valh<<8) + vall) * 0.1; // unit is 0.1 uW
		cval[chn] = 10 * log10f(val/1000);
		if (!mpod_scriptMode)
			printf("[%2d] %04.1f \tuW %02.2f dBm\n", chn, val, cval[chn]);
		else
			printf("%04.1f,%02.2f,", val, cval[chn]);
		reg=reg-2;
	}//for
	if (mpod_scriptMode) printf("\n"); // one minipod per line
	*data = cval;
	return(0);
}
/***********************************************************************************//**
 * \fn int mpod_lightOutput(int dev, int mpid, float **data)
 * \brief Function to get and print all light output values of one TX minipod
 *
 * \param mpid: the target minipod component 1, 3,  5 or 7
 * \param data: a pointer at which the function will put an allocated array of 11 values
 *              (chn 0 to 11) that the user will have to freed after use.
 *              value returned in the array are in uW
 * \return 0 success, -1 error
 *//***********************************************************************************/
int mpod_lightOutput(int dev, int mpid, float **data){
	char regMsg[] = "light output power";
	
	return mpod_dumpLightpower(dev, mpid,MPOD_LIGHTOUTLSB00,regMsg,mpod_isTX,data);
}
/***********************************************************************************//**
 * \fn int mpod_getLightOutput(int dev, int mpid, float *data)
 * \brief Function to get and print all light output values of one TX minipod in python
 *
 * \param mpid: the target minipod component 1, 3,  5 or 7
 * \param data: a pointer to an already allocated allocated array of 12 values
 *              value returned in the array are in uW
 * \return 0 success, -1 error
 *//***********************************************************************************/
int mpod_getLightOutput(int dev, int mpid, float *data){
	return mpod_get12FloatData(dev, mpod_lightOutput, mpid, data);
}
/***********************************************************************************//**
 * \fn int mpod_lightInput(int dev, int mpid, float **data)
 * \brief Function to get and print all optical power values of one RX minipod
 *
 * \param mpid: the target minipod component 0, 2 , 4 or 6
 * \param data: a pointer at which the function will put an allocated array of 11 values
 *              (chn 0 to 11) that the user will have to freed after use.
 *              value returned in the array are in dBm
 * \return 0 success, -1 error
 *//***********************************************************************************/
int mpod_lightInput(int dev, int mpid, float **data){
	char regMsg[] = "light input power PAVE";
	
	return mpod_dumpLightpower(dev,mpid,MPOD_OPPOWERLSB00,regMsg,mpod_isRX,data);
}
/***********************************************************************************//**
 * \fn int mpod_getLightInput(int dev, int mpid, float *data)
 * \brief Function to get and print all light input values of one RX minipod in python
 *
 * \param mpid: the target minipod component 0, 2 , 4 or 6
 * \param data: a pointer to an already allocated allocated array of 12 values
 *              value returned in the array are in uW
 * \return 0 success, -1 error
 *//***********************************************************************************/
int mpod_getLightInput(int dev, int mpid, float *data){
	return mpod_get12FloatData(dev, mpod_lightInput, mpid, data);
}
/*****************************************************************************************//**
 * \fn int mpod_elapsedPowerOn(int dev, int mpid, int *dur)
 * \brief Function to get the elapsed operating time of one minipod in hours (2h resolution)
 *
 *  Comment: this data seem not implemeneted in the chips while documented.
 *
 * \param mpid: the target minipod component 0 to 7
 * \param  dur: a pointer at which the function will put the read value.
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_elapsedPowerOn(int dev, int mpid, int *dur){
	unsigned valh, vall;
	
	// see how the target device is specified ... compute target device address
	if (mpod_read(dev, mpid, MPOD_ELAPSEREGMSB, &valh)){
		printf("ERROR: MPOD can't read the power elapse register on i2c bus");
		return(-1);
	}
	mpod_read(dev, mpid, MPOD_ELAPSEREGLSB , &vall);
	*dur = 2*((valh<<8) + vall);
	if (!mpod_scriptMode)
		printf("Component %d is running since %d Hours (2H granularity)\n", mpid, *dur);
	else
		printf("%d",*dur); // in hours 2H granularity
		
	return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_reset(int dev, int mpid)
 * \brief Function to reset all registers values to their default factory values.
 *
 * \param mpid: the target minipod component 0 to 7
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_reset(int dev, int mpid){
	
	if (mpod_write(dev, mpid, MPOD_RESET  , 1)){
			printf("ERROR: can't reset minipod %d board %d\n", mpid, dev);
			return(-1);
		}
	printf("Reset done on minipod %d board %d\n", mpid, dev);
	return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_channelDisable (int dev, int mpid, int channel, int flag)
 * \brief Function to disable one channel of one minipod.
 *
 * \param mpid   : the target minipod component 0 to 7
 * \param channel: the target channel 0 to 12
 * \param flag   : 0=enable  1=disable
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_channelDisable(int dev, int mpid, int channel, int flag){

char regMsg[]="channel disable";
return (mpod_setBitIn12bitsOn2words(dev, mpid, channel, flag, MPOD_CHDISABLE1, MPOD_CHDISABLE2,
									regMsg, NULL));
}
/*****************************************************************************************//**
 * \fn int mpod_channelDisDumpAll (int dev, int mpid)
 * \brief Function to dump the disable status of all channels of one minipod.
 *
 * \param mpid   : the target minipod component 0 to 7
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_channelDisDumpAll(int dev, int mpid){
char regName[]="Disable status enable/disable";

return mpod_dumpAllBitIn12bitsOn2words(dev, mpid, 
											MPOD_CHDISABLE1, MPOD_CHDISABLE2,
											regName,
											NULL,
											'e','d');
}
/*****************************************************************************************//**
 * \fn int mpod_polarityFlip (int dev, int mpid, int channel, int flag)
 * \brief Function to flip input/output polarity one channel of one minipod.
 *
 * \param mpid   : the target minipod component 0 to 7
 * \param channel: the target channel 0 to 12
 * \param flag   : 0=enable  1=disable
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_polarityFlip(int dev, int mpid, int channel, int flag){

char regMsg[]="polarity flipped";
int res;

mpod_pageSelect(dev, mpid , 1) ;
res = mpod_setBitIn12bitsOn2words(dev, mpid, channel, flag, MPOD_POLARFLIP1, MPOD_POLARFLIP2,
									regMsg, NULL);
mpod_pageSelect(dev, mpid , 0) ;
return res;
}
/*****************************************************************************************//**
 * \fn int mpod_channelPolarFlipDumpAll (int dev, int mpid)
 * \brief Function to dump the input/output polarity status of all channels of one minipod.
 *
 * \param mpid   : the target minipod component 0 to 7
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_channelPolarFlipDumpAll(int dev, int mpid){
char regName[]="Polarity flip status";
int res;

mpod_pageSelect(dev, mpid , 1) ;
res = mpod_dumpAllBitIn12bitsOn2words(dev, mpid, 
											MPOD_POLARFLIP1, MPOD_POLARFLIP2,
											regName,
											NULL,
											'0','1');
mpod_pageSelect(dev, mpid , 0) ;
return res;
}
/*****************************************************************************************//**
 * \fn int mpod_marginActivation (int dev, int mpid, int channel, int flag)
 * \brief Function to acivate margin one channel of one minipod.
 *
 * \param mpid   : the target minipod component 0 to 7
 * \param channel: the target channel 0 to 12
 * \param flag   : 0=enable  1=disable
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_marginActivation(int dev, int mpid, int channel, int flag){

char regMsg[]="margin activation";
return (mpod_setBitIn12bitsOn2words(dev, mpid, channel, flag, MPOD_MARGINACT1, MPOD_MARGINACT2,
									regMsg, mpod_isTX));
}
/*****************************************************************************************//**
 * \fn int mpod_marginActDumpAll (int dev, int mpid)
 * \brief Function to dump the margin activation status of all channels of one minipod.
 *
 * \param mpid   : the target minipod component 0 to 7
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_marginActDumpAll(int dev, int mpid){
char regName[]="margin activation/deactivation";
return mpod_dumpAllBitIn12bitsOn2words(dev, mpid, 
											MPOD_MARGINACT1, MPOD_MARGINACT2,
											regName,
											mpod_isRX,
											'd','a');
}
/*****************************************************************************************//**
 * \fn int mpod_readPatQuartMulti(int dev, int mpid, unsigned **data, int higherReg, char *idChain)
 * \brief Function general to read the 12 channels 4 bits codes of a minipod when they are 
 *  stored 2 by 2 (msb/lsb) in 6 registers (reverse order higher channel in lower registers) 
 *
 * It is used to factorize code for VOD, deamphasis and equalization read/write global or single
 *
 * \param mpid: the target RX minipod component 0, 2, 4 and 6 (RX)
 * \param data: a pointer at which the function will put an allocated array of 12 code values
 *              (chn 0 to 11) that the user will have to freed after use.
 * \param higherReg : the higher register number (for channel 0 and 1)
 * \param idChain: an identification chain to put in error message
 * \return 0 success, -1 error
 *//****************************************************************************************/
static int mpod_readPatQuartMulti(int dev, int mpid, int **data, int higherReg, char *idChain){
	int reg, i, msb, lsb, chn;
	unsigned val;
	int     *res;
	
	mpod_pageSelect(dev, mpid , 1) ;
	if (!(res = (int *)malloc(12*sizeof(int)))){
		printf("ERROR: Can't malloc memory for  amplitude control data\n");
		return(-1);
	}
	/* All the output amplitude control registers for all channels are in sequence with two channels per reg
	   starting with channel 11-10 ending with 1-0  thus 6 conscecutive regs*/
	chn = 0;
	reg = higherReg;
	for (i=6; i>0; i--){
		if (mpod_read(dev, mpid, reg, &val)){
			printf("ERROR: MPOD can't read the %s register on i2c bus\n", idChain);
			return(-1);
		}
		msb = val/16;
		lsb = val & 0x0f;
		// store the codes to pass to the user
		res[chn]  =lsb;
		res[chn+1]=msb;
		chn=chn+2;
		reg = reg-1;
	}
	*data=res;
	mpod_pageSelect(dev, mpid , 0);
	return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_readPatQuartSingle(int dev, int mpid, int channel, int *code, unsigned *places, char *idChain)
 * \brief Function to get the RX output amplitude control of one channel of a minipod
 *
 * \param mpid    : the target RX minipod component 0, 2, 4 and 6 (RX)
 * \param channel : the channel to read
 * \param data    : a pointer at which the function will put the returned code value.
 * \param places  : an array giving the register/places pairs for all channels
 * \param idChain : an identification chain to put in error message
 * \return 0 success, -1 error
 *//****************************************************************************************/
static int mpod_readPatQuartSingle(int dev, int mpid, int channel, int *code, quartPlace *places, char *idChain){
	int reg, qval;
	unsigned val;
	
	mpod_pageSelect(dev, mpid , 1) ;
	reg = places[channel].reg; 
	if (mpod_read(dev, mpid, reg, &val)){
			printf("ERROR: MPOD can't read %s register on i2c bus\n", idChain);
			return(-1);
	}
	// test low quarter is used and extract the proper quarter
	qval = (places[channel].quart==0) ? val & 0x0f : val/16;
	// qval = ((*places)[channel][1]==0) ? val & 0x0f : val/16;
	*code=qval;
	mpod_pageSelect(dev, mpid , 0);
	return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_writePatQuartMulti(int dev, int mpid, unsigned *data, int higherReg, char *idChain)
 
 * \brief Function general to write the 12 channels 4 bits codes of a minipod when they are 
 *  stored 2 by 2 (msb/lsb) in 6 registers (reverse order higher channel in lower registers) 
 *
 * It is used to factorize code for VOD, deamphasis and equalization read/write global or single
 *
 * \param mpid: the target RX minipod component 0, 2, 4 and 6 
 * \param data: a pointer to an array of 12 code values ordered from channel 0 to 11
 * \return 0 success, -1 error
 *//****************************************************************************************/
static int mpod_writePatQuartMulti(int dev, int mpid, int *data, int higherReg, char *idChain){
	int i, val, chn, reg;

	mpod_pageSelect(dev, mpid, 1) ;
	/* All the output amplitude control registers for all channels are in sequence with two channels per reg
	   starting with channel 11-10 ending with 1-0  thus 6 conscecutive regs*/
	reg = higherReg;
	chn = 0;
	for (i=6; i>0; i--){
		val = (data[chn+1]<<4) + data[chn];
		if (mpod_write(dev, mpid, reg, val)){
			printf("ERROR: can't write the RX %s register on i2c bus", idChain);
			return(-1);
		}
		chn = chn+2;
		reg = reg-1;
	}
	mpod_pageSelect(dev, mpid , 0);
	return(0);
}
/***************************************************************************************************//**
 * \fn int mpod_writePatQuartSingle(int dev, int mpid, int channel,  int code, quartPlaces *places, char *idChain)
 * \brief Function to set the RX output amplitude control of the a channel of a minipod
 *
 * Note this function is not efficient but we dont care we keep it simple!
 *
 * \param mpid    : the target RX minipod component 0, 2, 4 and 6
 * \param channel : the channel to set the value 
 * \param places  : the array describing the places of all channel and registers/codes associations
 * \param code    : the code value to write (0 to 7)
 * \param idChain : the string to identify the function in error messages
 * \return 0 success, -1 error
 *//***************************************************************************************************/
static int mpod_writePatQuartSingle(int dev, int mpid, int channel, int code, quartPlace *places, char *idChain){

	int val, reg;
	unsigned curData;

	mpod_pageSelect(dev, mpid , 1);
	reg = places[channel].reg;
	if (mpod_read(dev, mpid, reg, &curData)){
			printf("ERROR: MPOD can't read the %s register on i2c bus\n", idChain);
			return(-1);
	}
	if (places[channel].quart==0) {// test if low quarter is used 
		val = curData & 0xf0; //nullify used quarter
		val = val | code;
	}
	else {
		val = curData & 0xf; //nullify used quarter
		val  = val | (code<<4);
	}
	if (mpod_write(dev, mpid, reg, val)){
			printf("ERROR: MPOD can't write the RX %s register on i2c bus\n", idChain);
			return(-1);
		}
	mpod_pageSelect(dev, mpid , 0);
	return(0);
}
/*****************************************************************************************//**
 * \fn void mpod_printVOD(int dev, int mpid, unsigned *data)
 * \brief Function to print the RX output amplitude control of the 12 channels of a minipod
 *
 * \param mpid: the target RX minipod component 0, 2, 4 and 6 (RX)
 * \param data: a pointer at which the function receive an array with 12 code valuesb to print

 *//****************************************************************************************/
static void mpod_printVOD(int dev, int mpid, int *data){
	int i;
	if (!mpod_scriptMode){
		printf("VOD output amplitude for minipod %d board %d\n", mpid, dev);
		printf("---------------------------------------------\n");
		printf(" chn ! code! Min ! Nominal ! Max ! Units    !\n");
		printf("---------------------------------------------\n");
	}
	for (i=0; i<12; i++){
		if (!mpod_scriptMode)
			printf(" %3d ! %3d ! %3d !   %3d   ! %3d ! mVppd !\n", i, data[i],
					mpod_ampControlCodes[data[i]][0], mpod_ampControlCodes[data[i]][1], mpod_ampControlCodes[data[i]][2]);
		else
			printf("%3d,%3d,%3d,%3d,%3d,", i,  data[i], 
					mpod_ampControlCodes[data[i]][0], mpod_ampControlCodes[data[i]][1], mpod_ampControlCodes[data[i]][2]);
	}	
	if (!mpod_scriptMode)
		printf("------------------------------------------\n");
	else
		printf("\n");
}
/*****************************************************************************************//**
 * \fn void mpod_printVODSingle(int dev, int mpid, int channel, int code)
 * \brief Function to print the RX output amplitude control of one channel of a minipod
 *
 * \param mpid    : the target RX minipod component 0, 2, 4 and 6 (RX)
 * \param channel : the channel to print
 * \param data    : the code value of this channel to print.
 * \return 0 success, -1 error
 *//****************************************************************************************/
static void mpod_printVODSingle(int dev, int mpid, int channel, int code){
	if (!mpod_scriptMode){
		printf("VOD output amplitude for minipod %d channel %d board %d\n", mpid, channel, dev);
		printf("------------------------------------------\n");
		printf(" chn ! code! Min ! Nominal ! Max ! Units !\n");
		printf("------------------------------------------\n");
	}
	if (!mpod_scriptMode)
		printf(" %3d ! %3d ! %3d !   %3d   ! %3d ! mVppd !\n", channel,  code,
					mpod_ampControlCodes[code][0], mpod_ampControlCodes[code][1], mpod_ampControlCodes[code][2]);
	else
		printf("%3d,%3d,%3d,%3d,%3d,", channel,  code,
					mpod_ampControlCodes[code][0], mpod_ampControlCodes[code][1], mpod_ampControlCodes[code][2]);
	if (!mpod_scriptMode)
		printf("------------------------------------------\n");
	else
		printf("\n");
}
/*****************************************************************************************//**
 * \fn int mpod_readVOD(int dev, int mpid, unsigned **data)
 * \brief Function to get the RX output amplitude control of the 12 channels of a minipod
 *
 * \param mpid: the target RX minipod component 0, 2, 4 and 6 (RX)
 * \param data: a pointer at which the function will put an allocated array of 12 code values
 *              (chn 0 to 11) that the user will have to freed after use.
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_readVOD(int dev, int mpid, int **data){
	char subject[] = "VOD amplitude control";

	// check RX
	if (!(mpod_isRX(mpid))){
		printf("ERROR: minpod %d is not an RX\n", mpid);
		return(-1);
	};
	// get the data
	if (mpod_readPatQuartMulti(dev, mpid, data, MPOD_RXVOD1, subject)==-1)
		return(-1);
	// print the data
	mpod_printVOD(dev, mpid, *data);
	return(0);
}

/*****************************************************************************************//**
 * \fn int mpod_getVOD(int dev, int mpid, unsigned *data)
 * \brief Function to get the RX output amplitude control of the 12 channels of a minipod in python
 *
 * Function to avoid to have to manage dynamically allocated arrays in the python wrapper.
 *
 * \param mpid: the target RX minipod component 0, 2, 4 and 6 (RX)
 * \param data: a pointer to an already allocated array of 12 code values
 *              (chn 0 to 11) that the user will have to freed after use.
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_getVOD(int dev, int mpid, int *data){
	return mpod_get12IntData(dev, mpod_readVOD,mpid, data);
}
/*****************************************************************************************//**
 * \fn int mpod_readVODSingle(int dev, int mpid, int channel, int *code)
 * \brief Function to get the RX output amplitude control of one channel of a minipod
 *
 * \param mpid    : the target RX minipod component 0, 2, 4 and 6 (RX)
 * \param channel : the channel to read
 * \param data    : a pointer at which the function will put the returned code value.
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_readVODSingle(int dev, int mpid, int channel, int *code){
	char subject[] = "VOD amplitude control";
	
	// check RX
	if (!(mpod_isRX(mpid))){
		printf("ERROR: minpod %d is not an RX\n", mpid);
		return(-1);
	};
	// get the data
	if (mpod_readPatQuartSingle(dev, mpid, channel, code, mpod_vodQuarters, subject)==-1)
		return(-1);
	// print the data
	mpod_printVODSingle(dev, mpid, channel, *code);
	return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_writeVOD(int dev, int mpid, unsigned *data)
 * \brief Function to set the RX output amplitude control of the 12 channels of a minipod
 *
 * \param mpid: the target RX minipod component 0, 2, 4 and 6 
 * \param data: a pointer to an array of 12 code values ordered from channel 0 to 11
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_writeVOD(int dev, int mpid, int *data){
	char subject[] = "VOD amplitude control";
	
	// check RX
	if (!(mpod_isRX(mpid))){
		printf("ERROR: minpod %d is not an RX\n", mpid);
		return(-1);
	};
	// write the data
	mpod_writePatQuartMulti(dev, mpid, data, MPOD_RXVOD1, subject);
	return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_writeVODSingle(int dev, int mpid, int channel, int code)
 * \brief Function to set the RX output amplitude control of the a channel of a minipod
 *
 * Note this function is not efficient but we dont care we keep it simple!
 *
 * \param mpid    : the target RX minipod component 0, 2, 4 and 6
 * \param channel : the channel to set the value 
 * \param data    : the code value to write (0 to 7)
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_writeVODSingle(int dev, int mpid, int channel, int code){
	char subject[] = "VOD amplitude control";
	
	// check RX
	if (!(mpod_isRX(mpid))){
		printf("ERROR: minipod %d is not an RX\n", mpid);
		return(-1);
	};
	// check code
	if (code>7){
		printf("ERROR: RX output amplitude control register code should be less than 0111b\n");
		return(-1);
	}
	// write the data
	mpod_writePatQuartSingle(dev, mpid, channel, code, mpod_vodQuarters, subject);
	return(0);
}
/*****************************************************************************************//**
 * \fn void mpod_printDeamphasis(int dev, int mpid, unsigned *data)
 * \brief Function to print the RX output deamphasis codes of the 12 channels of a minipod
 *
 * \param mpid: the target RX minipod component 0, 2, 4 and 6 (RX)
 * \param data: a pointer at which the function receive an array with 12 code values to print

 *//****************************************************************************************/
static void mpod_printDeamphasis(int dev, int mpid, int *data){
	int i;
	if (!mpod_scriptMode){
		printf("output deamphasis for minipod %d board %d\n", mpid, dev);
		printf("------------------------\n");
		printf(" chn ! code !Deamphasis!\n");
		printf("------------------------\n");
	}
	for (i=0; i<12; i++){
		if (!mpod_scriptMode)
			printf(" %3d !  %3d !  %3d%c    !\n", i,   data[i], (data[i]*100/7), '%');
		else
			printf("%3d,%3d,%3d%c,", i,   data[i], (data[i]*100/7), '%');
	}
	if (!mpod_scriptMode)
		printf("------------------------\n");
	else
		printf("\n");
}
/*****************************************************************************************//**
 * \fn void mpod_printDeamphasisSingle(int dev, int mpid, int channel, int code)
 * \brief Function to print the RX deamphasis code of one channel of a minipod
 *
 * \param mpid    : the target RX minipod component 0, 2, 4 and 6 (RX)
 * \param channel : the channel to print
 * \param data    : the code value of this channel to print.
 * \return 0 success, -1 error
 *//****************************************************************************************/
static void mpod_printDeamphasisSingle(int dev, int mpid, int channel, int code){	
	if (!mpod_scriptMode){						
		printf("output deamphasis for minipod %d board %d\n", mpid, dev);
		printf("------------------------\n");
		printf(" chn ! code !Deamphasis!\n");
		printf("------------------------\n");
	}
	if (!mpod_scriptMode)	
		printf(" %3d !  %3d !  %3d%c    !\n", channel,   code, (code*100/7), '%');
	else
		printf("%3d,%3d,%3d%c,", channel,   code, (code*100/7), '%');
	if (!mpod_scriptMode)
		printf("------------------------\n");
	else
		printf("\n");
}
/*****************************************************************************************//**
 * \fn int mpod_readDeamphasis(int dev, int mpid, int **data)
 * \brief Function to get the RX deamphasis control of the 12 channels of a minipod
 *
 * \param mpid: the target minipod component 0, 2, 4 or 6
 * \param data: a pointer at which the function will put an allocated array of 12 code values
 *              (chn 0 to 11) that the user will have to freed after use.
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_readDeamphasis(int dev, int mpid, int **data){
	char subject[] = "Deamphasis control";

	// check RX
	if (!(mpod_isRX(mpid))){
		printf("ERROR: minpod %d is not an RX\n", mpid);
		return(-1);
	};
	// get the data
	if (mpod_readPatQuartMulti(dev, mpid, data, MPOD_RXDEAMPH1, subject)==-1)
		return(-1);
	   
	// print the data
	mpod_printDeamphasis(dev, mpid, *data);
	return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_getDeamphasis(int dev, int mpid, int *data)
 * \brief Function to get the RX deamphasis control of the 12 channels of a minipod in python
 *
 * Function developped for the python wrapper to avoid manipulation of dynamically allocated
 * array of data in python.
 *
 * \param mpid: the target minipod component 0, 2, 4 or 6
 * \param data: a pointer to an already allocated array of 12 code values
 *              (chn 0 to 11) that the user will have to freed after use.
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_getDeamphasis(int dev, int mpid, int *data){
	return mpod_get12IntData(dev, mpod_readDeamphasis,mpid, data);
}
/*****************************************************************************************//**
 * \fn int mpod_readDeamphasisSingle(int dev, int mpid, int channel, int *code)
 * \brief Function to get the RX deamphasis control of the one channel of a minipod
 *
 * \param mpid    : the target minipod component 0, 2, 4 or 6
 * \param channel : the channel to read
 * \param data    : a pointer at which the function will put the read code value
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_readDeamphasisSingle(int dev, int mpid, int channel, int *code){
	char subject[] = "Deamphasis control";
	
	// check RX
	if (!(mpod_isRX(mpid))){
		printf("ERROR: minpod %d is not an RX\n", mpid);
		return(-1);
	};
	// get the data
	if (mpod_readPatQuartSingle(dev, mpid, channel, code, mpod_deamphQuarters, subject)==-1)
		return(-1);

	// print the value
	mpod_printDeamphasisSingle(dev, mpid, channel, *code);
	return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_writeDeamphasis(int dev, int mpid, int *data)
 * \brief Function to set the RX deamphasis control of the 12 channels of a minipod
 *
 * \param mpid: the target minipod component 0,2, 4 or 6
 * \param data: a pointer to an array of 11 code values ordered from channel 0 to 11
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_writeDeamphasis(int dev, int mpid, int *data){
	char subject[] = "Deamphasis control";
	
	// check RX
	if (!(mpod_isRX(mpid))){
		printf("ERROR: minpod %d is not an RX\n", mpid);
		return(-1);
	};
	// write the data
	mpod_writePatQuartMulti(dev, mpid, data, MPOD_RXDEAMPH1, subject);
	return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_writeDeamphasisSingle(int dev, int mpid, int channel, int code)
 * \brief Function to set the RX deamphasis control of one channels of one minipod
 *
 * \param mpid    : the target minipod component 0,2,4 or 6
 * \param channel : the target channel
 * \param data    : the code value to write
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_writeDeamphasisSingle(int dev, int mpid, int channel, int code){
	char subject[] = "Deamphasis control";
	
	// check RX
	if (!(mpod_isRX(mpid))){
		printf("ERROR: minpod %d is not an RX\n", mpid);
		return(-1);
	}
	// check code
	if (code>7){
		printf("ERROR: RX deamphasis register code should be less than 0111b");
		return(-1);
	}
	mpod_writePatQuartSingle(dev, mpid, channel, code, mpod_deamphQuarters, subject);
	return(0);
}
/*****************************************************************************************//**
 * \fn void mpod_printEqualization(int dev, int mpid, unsigned *data)
 * \brief Function to print the TX equalization codes of the 12 channels of a minipod
 *
 * \param mpid: the target TX minipod component 0, 2 and 6 (TX)
 * \param data: a pointer at which the function receive an array with 12 code values to print

 *//****************************************************************************************/
static int mpod_printEqualization(int dev, int mpid, int *data){
	int i;
   if (!(mpod_isTX(mpid))){
		printf("ERROR: Equalization only available for TX minipods\n");
		return(-1);
	};
	if (!mpod_scriptMode){
		printf("equalization for minipod %d borad %d\n", mpid, dev);
		printf("-----------------------------------------------\n");
		printf(" chn ! code! peak !midband\t!peak<>midband!\n");
		printf("-----------------------------------------------\n");
	}
	for (i=0; i<12; i++){
		if (!mpod_scriptMode)
			printf(" %3d ! %3d ! %2.2f !%2.2f\t! %2.2f\t     !\n", i,   data[i],
				mpod_equalizationCodes[data[i]][0], mpod_equalizationCodes[data[i]][1], mpod_equalizationCodes[data[i]][2]);
		else
			printf("%3d,%3d,%2.2f,%2.2f,%2.2f,", i,   data[i],
				mpod_equalizationCodes[data[i]][0], mpod_equalizationCodes[data[i]][1], mpod_equalizationCodes[data[i]][2]);
	}
	if (!mpod_scriptMode)
		printf("-----------------------------------------------\n");
	else
		printf("\n");
	return(0);
}
/*****************************************************************************************//**
 * \fn void mpod_printEqualizationSingle(int dev, int mpid, int channel, int code)
 * \brief Function to print the TX deamphasis code of one channel of a minipod
 *
 * \param mpid    : the target TX minipod component 1, 3, 5 and 7 
 * \param channel : the channel to print
 * \param data    : the code value of this channel to print.
 * \return 0 success, -1 error
 *//****************************************************************************************/
static int mpod_printEqualizationSingle(int dev, int mpid, int channel, int code){
	if (!(mpod_isTX(mpid))){
		printf("ERROR: Equalization only available for TX minipods\n");
		return(-1);
	};
	if (!mpod_scriptMode){							
		printf("equalization for minipod %d board %d\n", mpid, dev);
		printf("-----------------------------------------------\n");
		printf(" chn ! code! peak !midband\t!peak<>midband!\n");
		printf("-----------------------------------------------\n");
	}
	if (!mpod_scriptMode)
		printf(" %3d ! %3d ! %2.2f !%2.2f\t! %2.2f\t     !\n", channel,   code,
				mpod_equalizationCodes[code][0], mpod_equalizationCodes[code][1], mpod_equalizationCodes[code][2]);
	else
		printf("%3d,%3d,%2.2f,%2.2f,%2.2f,", channel,   code,
				mpod_equalizationCodes[code][0], mpod_equalizationCodes[code][1], mpod_equalizationCodes[code][2]);
	if (!mpod_scriptMode)
		printf("-----------------------------------------------\n");
	else
		printf("\n");
	return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_readEqualization(int dev, int mpid, int  **data)
 * \brief Function to get the TX Equalization control of the 12 channels of a minipod
 *
 * \param mpid: the target minipod component 0, 2 or 4
 * \param data: a pointer at which the function will put an allocated array of 12 code values
 *              (chn 0 to 11) that the user will have to freed after use.
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_readEqualization(int dev, int mpid, int **data){
	char subject[] = "equalization control";
	
	// check TX
	if (!(mpod_isTX(mpid))){
		printf("ERROR: minpod %d is not a TX\n", mpid);
		return(-1);
	}
	// get the data
	if (mpod_readPatQuartMulti(dev, mpid, data, MPOD_TXEQUALZ1, subject)==-1)
		return(-1);
	   
	// print the data
	mpod_printEqualization(dev, mpid, *data);
	return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_getEqualization(int dev, int mpid, int  *data)
 * \brief Function to get the TX Equalization control of the 12 channels of a minipod in python
 *
 * Function developped for the python wrapping to prevent the exchange of a dynamically allocated
 * array that should be free after use.

 * \param mpid: the target minipod component 0, 2 or 4
 * \param data: a pointer to an already allocated array of 12 code values
 *          
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_getEqualization(int dev, int mpid, int *data){
	return mpod_get12IntData(dev, mpod_readEqualization ,mpid, data);
}

/*****************************************************************************************//**
 * \fn int mpod_readEqualizationSingle(int dev, int mpid, int channel, int *code)
 * \brief Function to get the TX equalization control of the one channel of a minipod
 *
 * \param mpid    : the target minipod component 0, 2 or 4
 * \param channel : the channel to read
 * \param data    : a pointer at which the function will put the read code value
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_readEqualizationSingle(int dev, int mpid, int channel, int *code){
	char subject[] = "Equalization control";
	
	// check TX
	if (!(mpod_isTX(mpid))){
		printf("ERROR: minpod %d is not an TX\n", mpid);
		return(-1);
	};
	// get the data
	if (mpod_readPatQuartSingle(dev, mpid, channel, code, mpod_equalzQuarters, subject)==-1)
		return(-1);

	// print the value
	mpod_printEqualizationSingle(dev, mpid, channel, *code);
	return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_writeEqualization(int dev, int mpid, int *data)
 * \brief Function to set the TX Equalization control of the 12 channels of a minipod
 *
 * \param mpid: the target minipod component 0,2 or 4
 * \param data: a pointer to an array of 12 code values ordered from channel 0 to 11
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_writeEqualization(int dev, int mpid, int *data){
	char subject[] = "Equalization control";
	// check TX
	if (!(mpod_isTX(mpid))){
		printf("ERROR: minpod %d is not an TX\n", mpid);
		return(-1);
	}
	// write the data
	mpod_writePatQuartMulti(dev, mpid, data, MPOD_TXEQUALZ1, subject);
	return(0);
}
/*****************************************************************************************//**
 * \fn int mpod_writeEqualizationSingle(int dev, int mpid, int channel, int code)
 * \brief Function to set the TX equalization control of one channels of one minipod
 *
 * \param mpid    : the target minipod component 0,2 or 4
 * \param channel : the target channel
 * \param data    : the code value to write
 * \return 0 success, -1 error
 *//****************************************************************************************/
int mpod_writeEqualizationSingle(int dev, int mpid, int channel, int code){
	char subject[] = "Equalization control";
	
	// check TX
	if (!(mpod_isTX(mpid))){
		printf("ERROR: minpod %d is not an TX\n", mpid);
		return(-1);
	}
	// check code
	if (code>7){
		printf("ERROR: TX deamphasis register code should be less than 0111b");
		return(-1);
	}

	mpod_writePatQuartSingle(dev, mpid, channel, code, mpod_equalzQuarters, subject);
	return(0);
}
#ifdef MAIN_MINIPOD
/*==============================*/
/* main test program to execute */
/*==============================*/
int main(int argc, char **argv){
	int      i, mp, vcc, dur;
	float    *fdata = 0;
	int      *data, *data2;
	int      testVOD[]  ={0x04,0x4,0x4,0x4,0x4,0x4,0x4,0x04,0x4,0x4,0x4,0x4};	
	int      testCodes[] ={0x01,0x2,0x3,0x4,0x5,0x6,0x7,0x1,0x2,0x3,0x4,0x5};
	

	/* ATTENTION CHECK THE necessary bus clock init setting is done in the driver */
	if (mpod_init(0, 125000000, 100000))
		exit(2);

	///#I2C_PageSelect $targetDevice 1
	if (mpod_pageSelect(0, 3, 1)==-1) exit(2);
	///#I2C_PageSelect $targetDevice 0
	if (mpod_pageSelect(0, 3, 0)==-1) exit(2);
	
	
 ///  ========================== VENDOR DATA  ==========================
	for (mp=0; mp<6; mp++) {
		if (mpod_dumpVendorData(0,mp)==-1){
			printf("\nERROR: Can't print vendor data for minipod %d\n\n", mp);
			}
	}
	///  ========================== SQUELCH  ==========================
	mpod_squelchDumpAll(0,0);
	mpod_squelchDisableAll(0, 0, 0);
	mpod_squelchDumpAll(0, 0);
	mpod_squelchDisableAll(0, 0, 1);
	mpod_squelchDumpAll(0, 0);
	mpod_squelchDisable(0, 0 , 2 , 0);
	mpod_squelchDisable(0, 0 , 9 , 0);
	mpod_squelchDumpAll(0, 0);
	mpod_squelchDisable(0, 0 , 2 , 1);
	mpod_squelchDisable(0, 0 , 9 , 1);
	
	mpod_internalTempDumpAll(0);
	
	
	/// ========================== I2C_Internal33Vcc 3  ==========================
	if (mpod_internal33Vcc(0, 0, 3, &vcc))
		exit(2);
	else {
		printf("\tTEST Got 3.3 Vcc %d\n", vcc);
	}
	/// ========================== I2C_Internal25Vcc 3  ==========================
	if (mpod_internal25Vcc(0, 3, &vcc))
		exit(2);
	else {
		printf("\tTEST Got 2.5 Vcc %d\n", vcc);
	}
	
	/// ========================== I2C_lightInput 2  ==========================
	if (mpod_lightInput(0, 2, &fdata))
		exit(2);
	else {
		printf("\tTEST Got optical power\n");
		for (i=0;i<12;i++) 
			printf("\tTEST data[%d]=%2.2f dBm\n", i, fdata[i]);
		free(fdata);
	}
	for (mp=0; mp<6; mp++){
		if (mpod_lightInput(0, mp, &fdata))
			exit(2);
	}
	for (mp=0; mp<6; mp++){
		mpod_elapsedPowerOn(0, mp, &dur);
	}	
	
	//  ========================== VOD  ==========================
	// read store current values
	printf("READ-STORE CURRENT VOD\n");
	for (mp=0; mp<6; mp++){
		if (!(mp%2)) continue;
		if (mpod_readVOD(0, mp, &data))
			exit(2);
		else {
			for (i=0;i<12;i++) 
				printf("\tTEST vod[%d]=%d ", i, data[i]);
			printf("\n");
		}
	}
	// put new values
	printf("WRITE NEW VOD\n");
	for (mp=0; mp<6; mp++){
		if (!(mp%2)) continue;
		mpod_writeVOD(0, mp, testVOD);
	}
	// print to check
	printf("READ CHECK NEW VOD\n");
	for (mp=0; mp<6; mp++){
		if (!(mp%2)) continue;
		if (mpod_readVOD(0, mp, &data2))
			exit(2);
		else {
			for (i=0;i<12;i++) 
				printf("\tTEST vod[%d]=%d ", i, data2[i]);
			printf("\n");
			free(data2);	
		}
	}
	// restore saved values
	printf("READ CHECK NEW VOD\n");
	for (mp=0; mp<6; mp++){
		if (!(mp%2)) continue;
		mpod_writeVOD(0, mp, data);
	}
	free(data);
	// check again
	printf("RESTORE STORED VOD\n");
	for (mp=0; mp<6; mp++){
		if (!(mp%2)) continue;
		if (mpod_readVOD(0, mp, &data2))
			exit(2);
		else {
			for (i=0;i<12;i++) 
				printf("\tTEST vod[%d]=%d ", i, data2[i]);
			printf("\n");
			free(data2);	
		}
	}
	// ========================== DEAMPHASIS ===========================
	// read store current values
	printf("DUMP CURENT DEAMPHASIS\n");
	for (mp=0; mp<6; mp++){
		if (!(mp%2)) continue;
		if (mpod_readDeamphasis(0, mp, &data))
			exit(2);
		else {
			for (i=0;i<12;i++) 
				printf("\tTEST deamp[%d]=%d ", i, data[i]);
			printf("\n");
		}
	}
	// put new values
	printf("WRITE NEW DEAMPHASIS\n");
	for (mp=0; mp<6; mp++){
		if (!(mp%2)) continue;
		mpod_writeDeamphasis(0, mp, testCodes);
	}
	// print to check
	printf("READ TO CHECK DEAMPHASIS\n");
	for (mp=0; mp<6; mp++){
		if (!(mp%2)) continue;
		if (mpod_readDeamphasis(0, mp, &data2))
			exit(2);
		else {
			for (i=0;i<12;i++) 
				printf("\tTEST deamp[%d]=%d ", i, data2[i]);
			printf("\n");
			free(data2);	
		}
	}
	// restore saved values
	printf("RESTORE OLD DEAMPHASIS\n");
	for (mp=0; mp<6; mp++){
		if (!(mp%2)) continue;
		mpod_writeDeamphasis(0, mp, data);
	}
	free(data);
	// check again
	printf("READ TO CHECK AGAIN DEAMPHASIS\n");
	for (mp=0; mp<6; mp++){
		if (!(mp%2)) continue;
		if (mpod_readDeamphasis(0, mp, &data2))
			exit(2);
		else {
			for (i=0;i<12;i++) 
				printf("\tTEST deamp[%d]=%d ", i, data2[i]);
			printf("\n");
			free(data2);	
		}
	}
	
	//  ========================== EQUALIZATION  ==========================
	// read store current values
	printf("DUMP CURENT EQUALIZATION\n");
	for (mp=0; mp<6; mp++){
		if (mp%2) continue;
		if (mpod_readEqualization(0, mp, &data))
			exit(2);
		else {
			for (i=0;i<12;i++) 
				printf("\tTEST deamp[%d]=%d ", i, data[i]);
			printf("\n");
		}
	}
	// put new values
	printf("WRITE NEW EQUALIZATION\n");
	for (mp=0; mp<6; mp++){
		if (mp%2) continue;
		mpod_writeEqualization(0, mp, testCodes);
	}
	// print to check
	printf("READ TO CHECK EQUALIZATION\n");
	for (mp=0; mp<6; mp++){
		if (mp%2) continue;
		if (mpod_readEqualization(0, mp, &data2))
			exit(2);
		else {
			for (i=0;i<12;i++) 
				printf("\tTEST deamp[%d]=%d ", i, data2[i]);
			printf("\n");
			free(data2);	
		}
	}
	// restore saved values
	printf("RESTORE OLD EQUALIZATION\n");
	for (mp=0; mp<6; mp++){
		if (mp%2) continue;
		mpod_writeEqualization(0, mp, data);
	}
	free(data);
	// check again
	printf("READ TO CHECK AGAIN EQUALIZATION\n");
	for (mp=0; mp<6; mp++){
		if (mp%2) continue;
		if (mpod_readEqualization(0, mp, &data2))
			exit(2);
		else {
			for (i=0;i<12;i++) 
				printf("\tTEST deamp[%d]=%d ", i, data2[i]);
			printf("\n");
			free(data2);	
		}
	}
	
return(0);
}
#endif


