// File for Belle 2 DMA functions for PCIe40
#include "pcie40_b2dma.h"
#include "pcie40_ecs.h"
#include <stdio.h>
#include <string>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "systemConfig.h" 
#include "pcie40_s_pages.h" 
#include "altera_dma_cmd.h" 
#include <sys/mman.h>
#include <errno.h>
#include <cstring>
#include <unistd.h>


static int dma_file[ MAX_DEV ] ;
static unsigned int *dpp[ LINK_NUMBER ][ S_PAGES ] ;

// **********************************************************************************
// Open device for DMA
// **********************************************************************************
int dma_open( int fd ) {
  std::string devname ;
  struct dma_cmd G_dma_cmd;
  char cmdbuff[ 100 ] ;

  devname = "/dev/pcie40_" + std::to_string( (long long unsigned int) fd ) + "_dma" ;
  ssize_t f = open(devname.c_str(), O_RDWR);

  // map spage addresses
  for ( int i=0 ; i<LINK_NUMBER ; i++ )
    for ( int j=0 ; j<S_PAGES ; j++ ) {
      
      //set spage for mapping;
      unsigned act_spage = ( i << 8 ) | j ;
      G_dma_cmd.cmd  = ALTERA_CMD_ACTSPAGE ;
      G_dma_cmd.buf  = cmdbuff ;
      *(int *)cmdbuff = act_spage ;
      write(f, &G_dma_cmd, 0) ;

      dpp[ i ][ j ] = (unsigned int *) mmap( NULL , 
					     S_PAGE_SIZE ,  
					     PROT_READ | PROT_WRITE ,
					     MAP_SHARED, 
					     f, 
					     0 ) ;  //map spage
      
      if (dpp[i][j] == MAP_FAILED) {
        printf( "error: mmap: %s\n", std::strerror(errno));
        close ( f ) ; 
	return -1 ;
      }
    }

  dma_file[ fd ] = f ;
  return f ;
}

// **************************************************************************
// Close device for DMA
// **************************************************************************
int dma_close( int fd ) {
  for ( int i=0 ; i<LINK_NUMBER ; i++ )
    for ( int j=0 ; j<S_PAGES ; j++ ) {
      munmap( dpp[ i ][ j ] , S_PAGE_SIZE ) ;
    }
  close( dma_file[ fd ] ) ; 
  dma_file[ fd ] = -1 ; 
  return 0 ;
}

// ***************************************************************************
// DMA initialize
// ***************************************************************************
int dma_initialize( int fd ) {
  ioctl( dma_file[ fd ] , ALTERA_IOCX_CLEAR);
  return 0 ;
}

// ***************************************************************************
// Reset the spages memory
// ***************************************************************************
int pcie40_resetSPages( int fd ) {
  for ( int i=0 ; i<LINK_NUMBER ; i++ )
    for ( int j=0 ; j<S_PAGES ; j++ )
      memset( dpp[i][j] , 0 , S_PAGE_SIZE ) ;
  return 0 ;
}

// **************************************************************************
// Get pointer to a super page
// **************************************************************************
unsigned int * pcie40_getSuperPagePointer( int link , int spage , int slot ) {
  if ( ( link >= LINK_NUMBER ) || ( spage >= S_PAGES ) || ( slot >= S_PAGE_SLOT_NMB ) ) 
    return 0 ;
  return &dpp[ link ][ spage ][ slot * S_PAGE_SLOT_SIZE/4 ] ;
}

// **************************************************************************
// Get pointer to a super page
// **************************************************************************
unsigned int * pcie40_getSuperPageCopy( int link , int spage , int slot ) {
  if ( ( link >= LINK_NUMBER ) || ( spage >= S_PAGES ) || ( slot >= S_PAGE_SLOT_NMB ) ) 
    return 0 ;
  unsigned int * pointer = &dpp[ link ][ spage ][ slot * S_PAGE_SLOT_SIZE/4 ] ;
  unsigned int * newdata = new unsigned int[ S_PAGE_SLOT_SIZE/4 ] ;
  memcpy( newdata , pointer , S_PAGE_SLOT_SIZE ) ;
  return newdata ;
}

// **************************************************************************
// Free the superpage when it has been read
// **************************************************************************
int pcie40_freeSuperPage( int fd , int spage ) {
  UAR uar ;
  uar.sr.spag_ready = spage ;
  ioctl(dma_file[ fd ], ALTERA_IOCX_RCONT,uar.arg);
  return 0 ;
}

// **************************************************************************
// Stop the DMA
// **************************************************************************
int pcie40_dmaStop(int fd) { 
  int ret = 0 ;
  ret = ecs_write( fd , DMA_BAR , DMA_CTRL , 0 ) ;
  return ret ;
}

// **************************************************************************
// Start the DMA
// **************************************************************************
int pcie40_dmaStart(int fd) { 
  int ret = 0 ;
  UAR uar ;
  ret = ecs_write( fd , DMA_BAR , DMA_CTRL , 0x81 ) ;
  ioctl( dma_file[ fd ] , ALTERA_IOCX_RSTART,&uar.arg); 
  return uar.sr.spag_ready ;
}

// **************************************************************************
// Reset the DMA
// **************************************************************************
int pcie40_dmaReset(int fd) { 
  int ret = 0 ;
  ret = ecs_write( fd , DMA_BAR , DMA_RST , 1 << DMA_RST_BIT ) ;
  return ret ;
}

// **************************************************************************
// Set readout DMA
// **************************************************************************
int pcie40_dmaSetReadout( int fd ) {
  int ret = 0 ;
  ret = ecs_write( fd , DMA_BAR , DMA_FIFO_0 , 0 ) ;
  return ret ;
}

// **************************************************************************
// Set size from header
// **************************************************************************
int pcie40_setSizeFromHeader( int fd ) {
  int ret = 0 ;
  ret = ecs_write( fd , DMA_BAR , DMA_FIFO_1 , 0 ) ;
  return ret ;
}

// **************************************************************************
// Disable size from header
// **************************************************************************
int pcie40_disableSizeFromHeader( int fd ) {
  int ret = 0 ;
  ret = ecs_write( fd , DMA_BAR , DMA_FIFO_1 , 1 ) ;
  return ret ;
}

// **************************************************************************
// Disable data generator
// **************************************************************************
int pcie40_enableGenerator(int fd) { 
  int ret = 0 ;
  int value ; 
  value = ecs_read( fd , LLI_BAR , REG_TRIG_FREE_RESET ) ; 
  ret = ecs_write( fd , LLI_BAR , REG_TRIG_FREE_RESET , ( 1 << DISABLE_GENERATOR_BIT ) | value ) ;
  return ret ;
}

// **************************************************************************
// Enable data generator
// **************************************************************************
int pcie40_disableGenerator(int fd) { 
  int ret = 0 ;
  int value ; 
  value = ecs_read( fd , LLI_BAR , REG_TRIG_FREE_RESET ) ; 
  ret = ecs_write( fd , LLI_BAR , REG_TRIG_FREE_RESET , ( ~( 1 << DISABLE_GENERATOR_BIT ) ) & value ) ;
  return ret ;
}

// **************************************************************************
// Enable external trigger for data generator
// **************************************************************************
int pcie40_enableExternalTrigger(int fd) { 
  int ret = 0 ;
  int value ; 
  value = ecs_read( fd , LLI_BAR , REG_TRIG_FREE_RESET ) ; 
  ret = ecs_write( fd , LLI_BAR , REG_TRIG_FREE_RESET , 
		   ( 1 << USE_EXT_TRIG_BIT ) | value ) ;
  return ret ;
}

// **************************************************************************
// Disable external trigger for data generator
// **************************************************************************
int pcie40_disableExternalTrigger(int fd) { 
  int ret = 0 ;
  int value ; 
  value = ecs_read( fd , LLI_BAR , REG_TRIG_FREE_RESET ) ; 
  ret = ecs_write( fd , LLI_BAR , REG_TRIG_FREE_RESET , 
		   ( ~( 1 << USE_EXT_TRIG_BIT ) ) & value ) ;
  return ret ;
}

// **************************************************************************
// Set Busy level (not sure it is used)
// **************************************************************************
int pcie40_setBusyLevel( int fd , unsigned int level ) {
  return ecs_write( fd , LLI_BAR , DMA_BUSY_LEVEL , level ) ;
}


// **************************************************************************
// Use data from fibers
// **************************************************************************
int pcie40_useDataFromGenerator( int fd ) { 
  int ret = 0 ;
  int value ;
  value = ecs_read( fd , LLI_BAR , REG_TRIG_FREE_RESET ) ;
  ret = ecs_write( fd , LLI_BAR , REG_TRIG_FREE_RESET , ( ~( 1 << USE_DATA_FROM_FIBERS_BIT ) ) & value ) ;
  return ret ;
}

// **************************************************************************
// Use free signal
// **************************************************************************
int pcie40_useFreeSignal( int fd ) { 
  int ret = 0 ;
  int value ;
  value = ecs_read( fd , LLI_BAR , REG_TRIG_FREE_RESET ) ;
  ret = ecs_write( fd , LLI_BAR , REG_TRIG_FREE_RESET , ( 1 << USE_FREE_BIT ) | value ) ;
  return ret ;
}

// **************************************************************************
// Ignore free signal
// **************************************************************************
int pcie40_ignoreFreeSignal( int fd ) { 
  int ret = 0 ;
  int value ;
  value = ecs_read( fd , LLI_BAR , REG_TRIG_FREE_RESET ) ;
  ret = ecs_write( fd , LLI_BAR , REG_TRIG_FREE_RESET , ( ~( 1 << USE_FREE_BIT )) & value ) ;
  return ret ;
}


// **************************************************************************
// Set trigger frequency of data generator
// **************************************************************************
int pcie40_setGeneratorTriggerFrequency( int fd , unsigned int freq ) {
  int ret = 0 ;
  int value ; 
  value = ecs_read( fd , LLI_BAR , REG_TRIG_FREE_RESET ) ; 
  ret = ecs_write( fd , LLI_BAR , REG_TRIG_FREE_RESET , ( ( 0xFF & freq ) | ( 0xFF00 & value ) ) ) ;
  return ret ;
}

// **************************************************************************
// Get trigger frequency of data generator
// **************************************************************************
int pcie40_getGeneratorTriggerFrequency( int fd ) {
  int value ;
  value = ecs_read( fd , LLI_BAR , REG_TRIG_FREE_RESET ) ;
  return ( value & 0xFF ) ;
}

// **************************************************************************
// Use data from generator
// **************************************************************************
int pcie40_useDataFromFibers( int fd ) {
  int ret = 0 ;
  int value ;
  value = ecs_read( fd , LLI_BAR , REG_TRIG_FREE_RESET ) ;
  ret = ecs_write( fd , LLI_BAR , REG_TRIG_FREE_RESET , ( 1 << USE_DATA_FROM_FIBERS_BIT ) | value ) ;
  return 0 ;
}

// **************************************************************************
// number of words in event (for data generator)
// **************************************************************************
int pcie40_setNbWordInEvent( int fd , unsigned int n ) {
  int ret = 0 ;
  ret = ecs_write( fd , LLI_BAR , REG_NB_WORD_IN_EVENT , n & 0xFF ) ;
  return ret ;
}

// **************************************************************************
// number of words in event (for data generator)
// **************************************************************************
unsigned int pcie40_getNbWordInEvent( int fd ) {
  return ( ecs_read( fd , LLI_BAR , REG_NB_WORD_IN_EVENT ) & 0xFF ) ;
}

// **************************************************************************
// Number of events (internal data generator)
// **************************************************************************
int pcie40_setNbEvents( int fd , unsigned int n ) {
  int ret = 0 ;
  ret = ecs_write( fd , LLI_BAR , REG_NB_EVENTS , n & 0xFFFFFFFF ) ;
  return ret ;
}

// **************************************************************************
// Get number of events (internal data generator)
// **************************************************************************
unsigned int pcie40_getNbEvents( int fd ) {
  return ecs_read( fd , LLI_BAR , REG_NB_EVENTS ) ;
}

// **************************************************************************
// Reset the address of the DMA buffer
// **************************************************************************
int pcie40_b2dmabufferreset(int fd) { 
  return 0 ;
}

// **************************************************************************
// Resynchronize the b2link, returns 1 if success, 0 if error 
// **************************************************************************
int pcie40_b2dmabufferread(int fd , unsigned int result[] ) { 
  printf( "ERROR: not yet implemented for new DMA\n" ) ;
  return 1 ;
}

// **************************************************************************
// Resynchronize the b2link, returns 1 if success, 0 if error 
// **************************************************************************
int pcie40_b2dmapointerread(int fd , unsigned int ** pData ) { 
  return 1 ;
}

